#ifndef _HUD_H
#define _HUD_H

#include <list>

#include <graphics/Object2.h>
#include <graphics/Font.h>
#include <graphics/SDLMutex.h>
#include <graphics/Sounds.h>

#include "Vehicle.h"


GRAPHICS_BEGIN


// Das ist unsere Head-Up-Display-Klasse.
class HUD :	public Object2
{
protected:
	struct RadarTarget
	{
		int_t	type;
		float	x,
				z,
				alpha;
	};

	class Message
	{
	public:
		string				text;
		Texture::Reference	texture;

		inline Message(const string &_text) :
			text(_text)
		{	/* nothing to do */	}
		inline Message(const string &_text, Texture::Reference &_texture) :
			text(_text),
			texture(_texture)
		{	/* nothing to do */	}
	};


	Vehicle			*pVehicle;

	// Waffen
	int						m_int_weaponIndex;
	vector<const Weapon>	vWeapons;
	SDLMutex				mWeapons;

	// Radar
	float				m_fl_radarRange;
	list<RadarTarget>	lRadarObjects;
	SDLMutex			mRadarObjects;


	// Farbindizes fr die einzelnen Schielde.
	uint_t m_int_shieldColorIndex[6];

	// Farbwerte fr die Schielde.		
	float m_fl_shieldColors[11][3];		

	// Aktuelle Motorleistung in Prozent.
	float m_fl_performance;

	// Aktuelle Geschwinigkeit.
	float m_fl_velocity;

	// Atkuelle Tiefe.
	float m_fl_depth;

	// Aktueller Transmission-Text.
	Sound			sndTransmission;
	list<Message>	transmissions;
	SDLMutex		mTransmissions;
	float			fTransmissionLife;

	
	Font::Reference		rFont;

	// Grafiken des HUD.
	Texture::Reference	m_tex_shipTop,
						m_tex_shipBehind,
						m_tex_shield,
						m_tex_performance,
						m_tex_scala,
						m_tex_radar,
						m_tex_scan,
						m_tex_echo,
						m_tex_border,
						m_tex_borderMask,
						m_tex_glassBottom,
						m_tex_glassTop;


	virtual void	_load();
	virtual void	_remove();

	virtual void	_glLoad();
	virtual void	_glRemove();

	virtual void	_glRender();


public:
	// Unser HUD-Constructor
	HUD();


	// Diese Funktionen ndern private HUD-Daten.
    inline void	font(Font::Reference &_font)
	{	rFont = _font;	}
	void		skin(const string &strPath);

	inline void	vehicle(Vehicle *_pVehicle)
	{	pVehicle = _pVehicle;	}
	inline void	radarRange(const float &range)
	{	m_fl_radarRange = range;	}

	void		message(const char *text);
	void		message(const char *text, Texture::Reference texture);
	void		message(const string &text);
	void		message(const string &text, Texture::Reference texture);


	virtual bool	advance(float time);


	virtual void	glReload();
};


GRAPHICS_END
#endif
