#ifndef __PROJECTILE_H__INCLUDED__
#define __PROJECTILE_H__INCLUDED__

#include <vector>

#include <graphics/Sphere.h>
#include <graphics/Sounds.h>

#include <aquawars/ObjectTypes.h>


using namespace graphics;


class Vehicle;


// Das ist unsere Projectile-Klasse.
class Projectile :	public Sphere
{
protected:
	static Sound	sndBullet0,
					sndBullet1,
					sndBullet2,
					sndBullet3,
					sndBullet4,
					sndBullet5,
					sndBullet6,
					sndBullet7,
					sndBullet8,
					sndBullet9,
					sndExplosion0,
					sndExplosion1,
					sndExplosion2,
					sndExplosion3,
					sndExplosion4,
					sndExplosion5,
					sndExplosion6,
					sndExplosion7;

	Vehicle	*parent;
	float	fLifetime,
			fSpeed,
			fDamage;


	virtual void	impact(const Vector &position, bool terrain = false) = 0;


public:
	inline Projectile(Vehicle *_parent, const float &lifetime, const float &damage) :
		parent(_parent),
		Sphere(0.05f, 8, 4, true),
		fLifetime(lifetime),
		fDamage(damage)
	{	/* nothing to do */	}


	static void		addSounds();


	inline void		speed(const float &_speed)
	{	fSpeed = _speed;	}


	virtual bool	advance(float time);
};


class Bullet :	public Projectile
{
protected:
	static vector<Sound *>	impactSounds;


	virtual void		impact(const Vector &position, bool terrain = false);


public:
	inline Bullet(Vehicle *_parent, const float &lifetime, const float &damage) :
		Projectile(_parent, lifetime, damage)
	{	type(c_projectile, bullet);	}


	static inline void	addSound(Sound *pSound)
	{	impactSounds.push_back(pSound);	}
};


class GaussBullet :	public Projectile
{
protected:
	static vector<Sound *>	impactSounds;


	virtual void		impact(const Vector &position, bool terrain = false);


public:
	inline GaussBullet(Vehicle *_parent, const float &lifetime, const float &damage) :
		Projectile(_parent, lifetime, damage)
	{	type(c_projectile, gaussBullet);	}


	static inline void	addSound(Sound *pSound)
	{	impactSounds.push_back(pSound);	}
};


class Torpedo :	public Projectile
{
protected:
	static vector<Sound *>	impactSounds;


	virtual void		impact(const Vector &position, bool terrain = false);


public:
	inline Torpedo(Vehicle *_parent, const float &lifetime, const float &damage) :
		Projectile(_parent, lifetime, damage)
	{	type(c_projectile, torpedo);	}


	static inline void	addSound(Sound *pSound)
	{	impactSounds.push_back(pSound);	}
};


#endif
