#ifndef __VEHICLE_H__INCLUDED__
#define __VEHICLE_H__INCLUDED__

#include <vector>

#include <graphics/Group3.h>
#include <graphics/Model.h>

#include <graphics/Sounds.h>

#include "Weapon.h"


using namespace graphics;


class Vehicle :	public Group3
{
public:
	enum ArmourSegment
	{
		up,
		down,
		forward,
		backward,
		left,
		right
	};


protected:
	static Sound	sndExplosion0,
					sndExplosion1,
					sndExplosion2,
					sndExplosion3,
					sndExplosion4,
					sndExplosion5,
					sndExplosion6,
					sndExplosion7;

	Model		*pModel;

	// submarine properties
	const float	fMaxEnginePower,
				fMaxArmour;

	// current speed
	float		fArmour[6],
				fEnginePower,
				fPhiDiffNew,
				fThetaDiffNew,
				fPhiDiff,
				fThetaDiff,
				fPhi,
				fTheta,
				fSpeed,
				fCloudCoolDown,
				fBubbleCoolDown;

	// currently available weapons
	vector<Weapon>	weapons;
	// currently selected weapon
	size_t			nWeaponIndex;
	// fire weapon?
	bool			bFire,
					bDestroyed,
					bGod;


	virtual void	_load();
	virtual void	_remove();


public:
	Vehicle(	const string &strModel,
				const float &_fMaxEnginePower = 8.0f, const float &_fMaxArmour = 1000.0f,
				const float &_fScale = 5.0f													);


	// vehicle control (steering, power,...)
	inline void		direction(const float &horizontal, const float &vertical)
	{	fPhi = vertical;
		fTheta = horizontal;		}
	inline void		changeDirection(const float &horizontal, const float &vertical)
	{	fPhiDiffNew += vertical;
		fThetaDiffNew += horizontal;		}
	inline void		enginePower(const float &power)
	{	fEnginePower = power;	}

	inline float	getEnginePower() const
	{	return fEnginePower;	}
	inline float	getSpeed() const
	{	return fSpeed;			}


	// weapons control
	inline void	fire(const bool _fire)
	{	bFire = _fire;	}

	inline void	addWeapon(const Weapon::Type &weapon)
	{	weapons.push_back(Weapon(this, weapon));	}

	inline void	nextWeapon()
	{	nWeaponIndex = (nWeaponIndex + 1) % weapons.size();	}
	inline void	previousWeapon()
	{	if(nWeaponIndex)
			nWeaponIndex--;
		else
			nWeaponIndex = weapons.size() - 1;	}
	inline void	changeWeapon(int8_t weapon)
	{
		if(weapon >= 0)
			setWeapon(nWeaponIndex + weapon);
		else
			setWeapon(nWeaponIndex + weapons.size() - (-weapon) % weapons.size());	}
	inline void	setWeapon(size_t weapon)
	{	nWeaponIndex = weapon % weapons.size();	}

	inline void	refillWeapon(size_t weapon)
	{	if(weapon < weapons.size())
			weapons[weapon].refill();	}
	void		refillWeapons();

	inline const vector<Weapon> &	getWeapons() const
	{	return weapons;					}
	inline uint_t					getCurrentWeapon() const
	{	return (uint_t)nWeaponIndex;	}


	// god mode?
	inline void		god(bool enable = true)
	{	bGod = enable;	}
	inline void		toggleGod()
	{	bGod = !bGod;	}

	inline bool		isGod() const
	{	return bGod;	}


	// armour/shield control
	void			repair();

	inline bool		isDestroyed() const
	{	return bDestroyed;	}

	inline const float *	getArmour() const
	{	return fArmour;			}
	inline float			getMaxArmour() const
	{	return fMaxArmour;		}


	// event "callbacks"
	void			onHit(const Vector &where, const float &damage);
	virtual void	onTargetDestroyed(const Vehicle *pTarget);


	virtual bool	advance(float time);
};


#endif