#ifndef __FRAMEWORK__ARRAY_ALLOCATOR_H__INCLUDED__
#define __FRAMEWORK__ARRAY_ALLOCATOR_H__INCLUDED__

#include "_Allocator.h"


FRAMEWORK_BEGIN


/*
 * The _ArrayAllocatorBase creates and destroys arrays of Objects
 * using the new and delete operators. Additionally it provides
 * functions for copying and deleting the arrays.
 *
 * The static Allocator can't be copied or deleted!
 */
template <class ItemType, uint_t SIZE> class _ArrayAllocatorBase :
	public __AllocatorBase
{
public:
	typedef __AllocatorBase						_Base;
	typedef _ArrayAllocatorBase<ItemType, SIZE>	_Type;


public:
	inline _ArrayAllocatorBase<ItemType, SIZE>()
	{	/* nothing to do */	}

	ItemType *	_new() const
	{
		ItemType *pArray = new ItemType[SIZE];
		if(!pArray)
			throw OutOfMemoryError();
		return pArray;
	}
	ItemType *	_copy(const ItemType *_pArray) const
	{
		ItemType *pArray = new ItemType[SIZE];
		if(!pArray)
			throw OutOfMemoryError();

		for(uint_t i = 0; i < SIZE; i++)
			pArray[i] = _pArray[i];

		return pArray;
	}
	inline void	_delete(ItemType *pArray) const
	{	delete[] pArray;	}
};


/*
 * The Allocator class acts like the allocator base, and it can be
 * created using __new(), copied with __copy() and destroyed using
 * __delete().
 */
template <class Item, uint_t SIZE> class ArrayAllocator :
	public _ArrayAllocatorBase<Item, SIZE>
{
public:
	typedef _ArrayAllocatorBase<Item, SIZE>	_Base;
	typedef ArrayAllocator<Item, SIZE>			_Type;


	inline ArrayAllocator<Item, SIZE>()
	{	/* nothing to do */	}


	static inline _Type *	__new()
	{	return new _Type();	}
	_Type *					__copy() const
	{
		_Type *	pNew = new _Type;
		if(!pNew)
			throw OutOfMemoryError();
		return pNew;
	}
	inline void				__delete()
	{	delete this;	}
};


/*
 * The StaticArrayAllocator class acts like the allocator base, but it can't
 * be created using __new() or __copy() and destroyed using __delete() like
 * the ArrayAllocator.
 */
template <class Item, uint_t SIZE> class StaticArrayAllocator :
	public _ArrayAllocatorBase<Item, SIZE>,
	public _StaticAllocatorBase<StaticArrayAllocator<Item, SIZE> >
{
public:
	typedef _ArrayAllocatorBase<Item, SIZE>							_Base1;
	typedef _StaticAllocatorBase<StaticArrayAllocator<Item, SIZE> >	_Base2;
	typedef StaticArrayAllocator<Item, SIZE>						_Type;


	inline StaticArrayAllocator<Item, SIZE>()
	{	/* nothing to do */	}
};


FRAMEWORK_END
#endif
