#ifndef __FRAMEWORK__CLASS_MANAGER_H__INCLUDED__
#define __FRAMEWORK__CLASS_MANAGER_H__INCLUDED__

#include <map>

#include "Exception.h"
#include "Reference.h"
#include "IAllocator.h"


FRAMEWORK_BEGIN


template <	class Base,
			class Id,
			class ClassId,
			class Allocator = IAllocatorBase<Base, Id>,
			bool STATIC_ALLOCATOR = false				>
	class ClassManager :
		public __AllocatorBase,
		public	_StaticAllocatorBase
				<	ClassManager<Base, Id, ClassId, Allocator, STATIC_ALLOCATOR>	>
{
public:
	typedef __AllocatorBase
			_Base1;
	typedef _StaticAllocatorBase
			<	ClassManager<Base, Id, ClassId, Allocator, STATIC_ALLOCATOR>	>
			_Base2;
	typedef ClassManager<	Base,
							Id,
							ClassId,
							Allocator,
							STATIC_ALLOCATOR	>
			_Type;

	typedef Reference<	Base,
						Id,
						Allocator,
						STATIC_ALLOCATOR	>
			Reference;


protected:
	map<ClassId, Allocator *>	allocators;


	inline void	_add_dynamic(const ClassId &id, Allocator *pAllocator)
	{	if(!allocators.insert(map<ClassId, Allocator *>::value_type(id, pAllocator->__copy())).second)
			throw DuplicateKeyException();																}
	inline void	_add_static(const ClassId &id, Allocator *pAllocator)
	{	if(!allocators.insert(map<ClassId, Allocator *>::value_type(id, pAllocator)).second)
			throw DuplicateKeyException();													}


public:
	inline ClassManager<	Base,
							Id,
							ClassId,
							Allocator,
							STATIC_ALLOCATOR	>()
	{	/* nothing to do */	}
	inline ClassManager<	Base,
							Id,
							ClassId,
							Allocator,
							STATIC_ALLOCATOR	>(_Type &cm) :	allocators(cm.allocators)
	{	/* nothing to do */	}
	virtual ~ClassManager<	Base,
							Id,
							ClassId,
							Allocator,
							STATIC_ALLOCATOR	>()
	{	clear();	}


	inline void	add(const ClassId &id, Allocator *pAllocator)
	{
		if(STATIC_ALLOCATOR)
			_add_static(id, pAllocator);
		else
			_add_dynamic(id, pAllocator);
	}

	const Allocator *	get(const ClassId id) const
	{	
		map<ClassId, Allocator *>::const_iterator	it = allocators.find(id);
		if(it == allocators.end())
			throw InvalidKeyException();

		return it->second;
	}

	inline void	remove(const ClassId &id)
	{	allocators.erase(id);	}

	inline void	clear()
	{	allocators.clear();	}

	inline Reference	_new(const Id *pId) const
	{	const Allocator	*pAllocator = get((const ClassId)*pId);
		return Reference(pAllocator->_new(pId), pAllocator);	}
	void				_delete(Base *pItem) const
	{	delete pItem;	}

	inline Allocator *			operator[](const ClassId &id)
	{	return get(id);	}
	inline const Allocator *	operator[](const ClassId &id) const
	{	return get(id);	}
};


FRAMEWORK_END
#endif
