#ifndef __FRAMEWORK__EXCEPTION_H__INCLUDED__
#define __FRAMEWORK__EXCEPTION_H__INCLUDED__

#include <stdexcept>
#include <string>

#include "Reference.h"

#define DEFINE_EXCEPTION(exception_class, base_class, text)\
	class exception_class : public base_class\
	{\
	public:\
		inline exception_class(const string &strCause = string(), const string &strException = text) : base_class(strCause, strException) {}\
		inline exception_class(const Exception &eCause, const string strException = text) : base_class(eCause, strException) {}\
		inline exception_class & operator =(const exception_class &e)\
		{	base_class::operator=(e);\
			return *this;				}\
	}


FRAMEWORK_BEGIN


class Exception : public exception
{
protected:
	string					str,
							strException,
							strCause;
	Reference<Exception>	refCause;

	void	init();

public:
	Exception(const string &_strCause = string(), const string &_strException = "Exception");
	Exception(const Exception &eCause, const string &_strException = "Exception");

	Exception &	operator =(const Exception &e);

	inline	string							getExceptionString() const
	{	return strException;	}
	inline	string							getCauseString() const
	{	return strCause;		}
	inline	const Reference<Exception> &	getCauseReference() const
	{	return refCause;		}

	void					report() const;

	virtual const char *	what() const;

	inline	const string &	toString() const
	{	return str;	}
};


DEFINE_EXCEPTION(InvalidArgumentException, Exception, "Invalid Argument");
DEFINE_EXCEPTION(NullPointerException, Exception, "Null Pointer");
DEFINE_EXCEPTION(ArrayIndexOutOfBoundsException, Exception, "Array Index Out Of Bounds");
DEFINE_EXCEPTION(NotImplementedException, Exception, "Not Implemented");
DEFINE_EXCEPTION(NotSupportedException, Exception, "Not Supported");


DEFINE_EXCEPTION(IOException, Exception, "I/O Exception");
DEFINE_EXCEPTION(EndOfFileException, IOException, "End Of File");
DEFINE_EXCEPTION(TimeoutException, IOException, "Timeout");

DEFINE_EXCEPTION(KeyException, Exception, "Key Exception");
DEFINE_EXCEPTION(DuplicateKeyException, KeyException, "Duplicate Key");
DEFINE_EXCEPTION(InvalidKeyException, KeyException, "Invalid Key");


#ifdef _WINDOWS
string	getLastErrorString();
#endif


FRAMEWORK_END
#endif
