#ifndef __FRAMEWORK__IALLOCATOR_H__INCLUDED__
#define __FRAMEWORK__IALLOCATOR_H__INCLUDED__

#include "_Allocator.h"


FRAMEWORK_BEGIN


template <class Item, class Id = void> class IAllocatorBase :
	public __AllocatorBase
{
public:
	typedef __AllocatorBase				_Base;
	typedef	IAllocatorBase<Item, Id>	_Type;


public:
	inline IAllocatorBase<Item, Id>()
	{	/* nothing to do */	}
	virtual ~IAllocatorBase<Item, Id>()
	{	/* nothing to do */	}


	virtual Item *	_new() const = 0;
	virtual Item *	_new(const Id *pId) const = 0;
	virtual Item *	_copy(const Item *pItem) const = 0;
	virtual void	_delete(Item *pItem) const
	{	delete pItem;	}


	static inline  _Type *	__new()
	{	// can't compile Reference template without this
		throw NotSupportedError(getErrorMessage("can't create new *ABSTRACT* allocator"));	}
	virtual _Type *			__copy() const = 0;
	virtual void			__delete() = 0;
};


template <class Item, class Id = void> class IAllocator :
	public IAllocatorBase<Item, Id>
{
public:
	typedef IAllocatorBase<Item, Id>	_Base;
	typedef	IAllocator<Item, Id>		_Type;

	inline IAllocator<Item, Id>()
	{	/* nothing to do */	}

	virtual void	__delete()
	{	delete this;	}
};


template <class Item, class Id = void> class IStaticAllocator :
	public IAllocatorBase<Item, Id>,
	public _StaticAllocatorBase<IStaticAllocator<Item, Id> >
{
public:
	typedef IAllocatorBase<Item, Id>							_Base1;
	typedef _StaticAllocatorBase<IStaticAllocator<Item, Id> >	_Base2;
	typedef	IStaticAllocator<Item, Id>							_Type;


	inline IStaticAllocator<Item, Id>()
	{	/* nothing to do */	}


	// can't compile Reference template without this
	static inline _Type *	__new()
	{	return _Base2::__new();		}
	virtual _Type *			__copy() const
	{	return _Base2::__copy();	}
	virtual void			__delete()
	{	_Base2::__delete();	}
};


FRAMEWORK_END
#endif
