#ifndef __FRAMEWORK__OBJECT_MANAGER_H__INCLUDED__
#define __FRAMEWORK__OBJECT_MANAGER_H__INCLUDED__

#include <map>

#include "Exception.h"
#include "Reference.h"
#include "Allocator.h"


FRAMEWORK_BEGIN


template <	class Item,
			class Id,
			class Allocator = StaticAllocator<Item, Id>,
			bool STATIC_ALLOCATOR = true,
			class ResultAllocator = Allocator,
			bool STATIC_RESULT_ALLOCATOR = STATIC_ALLOCATOR	>
	class ObjectManager
{
public:
	typedef	ObjectManager<	Item,
							Id,
							Allocator,
							STATIC_ALLOCATOR,
							ResultAllocator,
							STATIC_RESULT_ALLOCATOR	>
			_Type;

	typedef	Reference<	Item,
						Id,
						ResultAllocator,
						STATIC_RESULT_ALLOCATOR	>
			Reference;

protected:
	Allocator			allocator;
	map<Id, Reference>	items;

public:
	inline ObjectManager<	Item,
							Id,
							Allocator,
							STATIC_ALLOCATOR,
							ResultAllocator,
							STATIC_RESULT_ALLOCATOR	>()
	{	/* nothing to do */	}
	inline ObjectManager<	Item,
							Id,
							Allocator,
							STATIC_ALLOCATOR,
							ResultAllocator,
							STATIC_RESULT_ALLOCATOR	>(_Type &om) :	items(om.items)
	{	/* nothing to do */	}
	virtual ~ObjectManager<	Item,
							Id,
							Allocator,
							STATIC_ALLOCATOR,
							ResultAllocator,
							STATIC_RESULT_ALLOCATOR	>()
	{	clear();	}

	inline void	add(const Id &id, Reference &item)
	{
		if(!items.insert(map<Id, Reference>::value_type(id, item)).second)
			throw DuplicateKeyException();
	}

	Reference	create(const Id &id)
	{
		// check if the map contains the requested item
		map<Id, Reference>::iterator	it = items.find(id);
		if(it != items.end())
			return it->second;

		// we have to create the item
		Reference	item(&allocator);
		item.create(&id);
		add(id, item);
		return item;
	}
	Reference	createReference(const Id &id)
	{
		// check if the map contains the requested item
		map<Id, Reference>::iterator	it = items.find(id);
		if(it != items.end())
			return it->second;

		// we have to create the item
		Reference	item(allocator._new(&id));
		add(id, item);
		return item;
	}

	Reference	get(const Id &id) const
	{
		map<Id, Reference>::iterator	it = items.find(id);
		if(it == items.end())
			throw InvalidKeyException();

		return it->second;
	}

	inline void	remove(const Id &id)
	{	items.erase(id);	}

	void		clean()
	{
		map<Id, Reference>::iterator	it = items.begin();

		while(it != items.end())
			if(it->second.getReferenceCount() <= 1)
				it = items.erase(it);
			else
				it++;
	}
	inline void	clear()
	{	items.clear();	}

	inline Reference	operator[](const Id &id)
	{	return create(id);	}
	inline Reference	operator[](const Id &id) const
	{	return get(id);		}
};


FRAMEWORK_END
#endif
