#ifndef __GRAPHICS__GL_ITEM_H__INCLUDED__
#define __GRAPHICS__GL_ITEM_H__INCLUDED__

#include <framework/Error.h>

#include "graphics.h"
#include "_gl.h"


GRAPHICS_BEGIN


class GlItem
{
protected:
	uint_t	nLoadCount,
			nGlCount;
	bool	bLoaded,
			bGlLoaded;


	// don't allow copying
	inline GlItem &	operator =(const GlItem &)
	{	throw NotImplementedError();	}

	virtual void	_load() = 0;
	virtual void	_remove() = 0;

	virtual void	_glLoad() = 0;
	virtual void	_glRemove() = 0;


public:
	inline GlItem()	:	nLoadCount(0),
						nGlCount(0),
						bLoaded(false),
						bGlLoaded(false)
	{	/* nothing to do */	}
	virtual	~GlItem();


	// returns how often load has been called
	// (without a subsequent call to remove)
	inline uint_t	getLoadCount() const
	{	return nLoadCount;	}
	// returns how often this item is references by OpenGL
	inline uint_t	getGlCount() const
	{	return nGlCount;	}


	// increases the load count and calls _load if necessary
	inline void	load()
	{	if(!nLoadCount && !bLoaded)
		{	_load();
			bLoaded = true;	}
		nLoadCount++;						}
	// decreases the load count and calls _remove if necessary
	inline void	remove()
	{	if(nLoadCount > 0)
			--nLoadCount;
		if(!nLoadCount && bLoaded)
		{	_remove();
			bLoaded = false;	}	}


	// increases the OpenGL reference count
	inline void		glAddReference()
	{	nGlCount++;	}
	// decreases the OpenGL reference count
	inline void		glRemoveReference()
	{	if(nGlCount > 0)
			nGlCount--;		}

	// verify that OpenGL has loaded this item
	void			glLoad();
	// specifies that OpenGL has been reset and the items
	// needs to be reloaded
	virtual void	glReload();
	// remove this item from OpenGL
	void			glRemove();

	// load this item and activates it (if necessary)
	virtual void	glBind();
	// bind this item (load if necessary)
	void			glBindLoad();
};


GRAPHICS_END
#endif
