#ifndef __GRAPHICS__MODEL_H__INCLUDED__
#define __GRAPHICS__MODEL_H__INCLUDED__

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#include "Group3.h"
#include "TransformationMatrix4.h"
#include "Vertex.h"
#include "Shape.h"
#include "Material.h"

/* MilkShape constants */
#define MS_MAX_NAME             32
#define MS_MAX_PATH             256

GRAPHICS_BEGIN

class Bone
{
public:
	struct KeyFrame
	{
		float	Time;
		float	Value[3];
	};

	typedef TransformationMatrix4<float>	Matrix;

	Bone();
	~Bone();
	bool	loadFromMs3dAsciiSegment( FILE *file );
	void	render( void );
	void	fixup( void );
	void	advanceTo( float time );
	void	initialize( void );
//	private:
	char	Name[MS_MAX_NAME];
	char	ParentName[MS_MAX_NAME];
	Bone	*Parent;						// pointer to parent bone (or NULL)
	float	startPosition[3];
	float	startRotation[3];
	Matrix	m_relative;				// fixed transformation matrix relative to parent 
	Matrix	m_final;				// absolute in accordance to animation
	int		NumPositionKeys;
	KeyFrame *PositionKeyFrames;
	int		NumRotationKeys;
	KeyFrame *RotationKeyFrames;
};

class Model : public Group3
{
public:
	typedef TransformationMatrix4<float>	Matrix;


protected:
	string	filename;
		
	float	MaxTime,		// length of animation (= Frames)
			CurrentTime;	// advancement within animation
	int		num_shapes,
			num_materials,
			num_bones;

	Shape		*shapes;
	Material	**materials;
	int			*material_indices;
	Bone		*bones;


	//bool    loadFromMs3dAsciiFile( const char *filename );
	void	_load();
	void	_remove();


public:
	Model(const string &filename);
	~Model();


	void	reloadTextures( void );
	void	renderShadow( float height );
	void	renderBones( void );
	bool	linkBones( void );
	void	initializeBones( void );
	void	attachSkin( void );


	virtual bool	advance(float time);

	virtual void	glReload();
};

GRAPHICS_END

#endif