#ifndef __GRAPHICS__OBJECT2_H__INCLUDED__
#define __GRAPHICS__OBJECT2_H__INCLUDED__

#include "Object.h"
#include "Renderable.h"


GRAPHICS_BEGIN


class _Object2 :	public Object,
					public Renderable
{
protected:
	typedef	_Vector2	Vector;
	typedef _Vector2D	VectorD;


public:
	inline _Object2() :	Object()
	{	/* nothing to do */	}


	// transformations
	inline void	preRotate(GLfloat phi)
	{	tmFinal.preRotateY(phi);	}
	inline void	postRotate(GLfloat phi)
	{	tmFinal.postRotateY(phi);	}

	inline void	preTranslate(const Vector &v)
	{	preTranslate(v.getX(), v.getY());	}
	inline void	postTranslate(const Vector &v)
	{	postTranslate(v.getX(), v.getY());	}
	inline void	preTranslate(const GLfloat &x, const GLfloat &y)
	{	tmFinal.preTranslate(x, y, 0.0f);	}
	inline void	postTranslate(const GLfloat &x, const GLfloat &y)
	{	tmFinal.postTranslate(x, y, 0.0f);	}

	inline void	preScale(const GLfloat &sx, const GLfloat &sy)
	{	tmFinal.preScale(sx, sy, 0.0f);	}
	inline void	postScale(const GLfloat &sx, const GLfloat &sy)
	{	tmFinal.postScale(sx, sy, 0.0f);	}


	// get functions
	inline void		getPosition(Vector &v) const
	{	v.set(	tmFinal.getElement(0, 3),
				tmFinal.getElement(1, 3)	);	}
	inline void		getPosition(VectorD &v) const
	{	v.set(	tmFinal.getElement(0, 3),
				tmFinal.getElement(1, 3)	);	}
	inline Vector	getPosition() const
	{	return Vector(	tmFinal.getElement(0, 3),
						tmFinal.getElement(1, 3)	);	}
	inline VectorD	getPositionD()
	{	return VectorD(	tmFinal.getElement(0, 3),
						tmFinal.getElement(1, 3)	);	}
	inline GLfloat	getPositionX() const
	{	return tmFinal.getElement(0, 3);	}
	inline GLfloat	getPositionY() const
	{	return tmFinal.getElement(1, 3);	}

	inline void		getLocalPosition(Vector &v) const
	{	v.set(	tmLocal.getElement(0, 3),
				tmLocal.getElement(1, 3)	);	}
	inline void		getLocalPosition(VectorD &v) const
	{	v.set(	tmLocal.getElement(0, 3),
				tmLocal.getElement(1, 3)	);	}
	inline Vector	getLocalPosition() const
	{	return Vector(	tmLocal.getElement(0, 3),
						tmLocal.getElement(1, 3)	);	}
	inline VectorD	getLocalPositionD()
	{	return VectorD(	tmLocal.getElement(0, 3),
						tmLocal.getElement(1, 3)	);	}
	inline GLfloat	getLocalPositionX() const
	{	return tmLocal.getElement(0, 3);	}
	inline GLfloat	getLocalPositionY() const
	{	return tmLocal.getElement(1, 3);	}

	inline void		getGlPosition(Vector &v) const
	{	v.set(	tmGL.getElement(0, 3),
				tmGL.getElement(1, 3)	);	}
	inline void		getGlPosition(VectorD &v) const
	{	v.set(	tmGL.getElement(0, 3),
				tmGL.getElement(1, 3)	);	}
	inline Vector	getGlPosition() const
	{	return Vector(	tmGL.getElement(0, 3),
						tmGL.getElement(1, 3)	);	}
	inline VectorD	getGlPositionD()
	{	return VectorD(	tmGL.getElement(0, 3),
						tmGL.getElement(1, 3)	);	}
	inline GLfloat	getGlPositionX() const
	{	return tmGL.getElement(0, 3);	}
	inline GLfloat	getGlPositionY() const
	{	return tmGL.getElement(1, 3);	}

	virtual void	glRemoveUnreferenced();
};


class Object2 :	public _Object2
{
protected:
	// render the object
	virtual void	_glRender() = 0;


public:
	inline Object2() :	_Object2()
	{	/* nothing to do */	}


	// load, bind and render the object
	virtual void	glRender();
};


GRAPHICS_END
#endif
