#ifndef __GRAPHICS__PARTICLELIST_H__
#define __GRAPHICS__PARTICLELIST_H__

#include <list>

#include "Group3.h"
#include "Particle.h"


GRAPHICS_BEGIN


class ParticleList :	public Group3
{
public:
	typedef	Vector3<float>	Vector;


protected:
	enum
	{
		EXPLOSION,
		FLASH,
		BUBBLE,
		SPARK,
		CLOUD
	};


	static Particle	m_par_explosion,
					m_par_bubble,
					m_par_spark,
					m_par_flash,
					m_par_cloud;


	ParticleList();


	// Gibt einen Zufallswert zurck, wobei die Wahrscheinlichkeit
	// fr einen kleinen Wert hher ist.
	static inline float	randomSize()
	{
		// Die Zufallswerte (zwischen 0 und 1) mit PI-halbe skalieren,
		// dann werden sie einer Cosinus-Funktion (= Verteilungskurve)
		// unterworfen und mit 2 multipliziert (= Skalierungseinheit der Gre).
		return (1.3f - pow(cos(((float)(rand() % 100 + 1)) / 100.0f * (float)(PI / 2.0f)), 0.2f));
	}


	// Funktionen um die unterschiedlichen Partikel in die Liste einzufgen.
	void insertBubbleParticle(const Vector &position);
	void insertBubbleParticle(const Vector &position, const Vector &direction, const float velocity);
	void insertSparkParticle(const Vector &position);
	void insertSparkParticle(const Vector &position, const Vector &direction, const float velocity);
	void insertCloudParticle(const Vector &position);
	void insertCloudParticle(const Vector &position, const Vector &direction, const float velocity);
	void insertFlashParticle(const Vector &position, const float size);
	void insertExplosionParticle(const Vector &position, const float size);

	// Erstellen von Partikeln, die sich in einer Explosion dreidimensional ausbreiten
	void insertExplosion3D(const int particleKind, const int number, const Vector &position, const float dispersionVelocity, const Vector &movementDirection);

public:		
	~ParticleList();

	// Erstellen von Partikeln, die sich in einer Explosion dreidimensional ausbreiten
	static ParticleList *	createCloudExplosion(const int number, const Vector &position, const float dispersionVelocity, const Vector &movementDirection = Vector(0.0f, 0.0f, 0.0f));

	static ParticleList *	createBubbleExplosion(const int number, const Vector &position, const float dispersionVelocity, const Vector &movementDirection = Vector(0.0f, 0.0f, 0.0f));


	// Erstellen einer 3D-Standardexplosion mit Bewegungsrichtung
	static ParticleList *	createExplosion3D(	const float explosionSize, const float flashSize, const float dispersion, const int bubbleNumber, const int sparkNumber, const Vector &position, const Vector &movementDirection = Vector(0.0f, 0.0f, 0.0f));


	static inline void	explosionTexture(Texture::Reference &newtexture)
	{	m_par_explosion.texture(newtexture);	}
	static inline void	bubbleTexture(Texture::Reference &newtexture)
	{	m_par_bubble.texture(newtexture);	}
	static inline void	sparkTexture(Texture::Reference &newtexture)
	{	m_par_spark.texture(newtexture);	}
	static inline void	flashTexture(Texture::Reference &newtexture)
	{	m_par_flash.texture(newtexture);	}
	static inline void	cloudTexture(Texture::Reference &newtexture)
	{	m_par_cloud.texture(newtexture);	}
};


GRAPHICS_END
#endif
