#ifndef __GRAPHICS__SDL_CONDITIONAL_H__INCLUDED__
#define __GRAPHICS__SDL_CONDITIONAL_H__INCLUDED__

#include <SDL_mutex.h>

#include <framework/Reference.h>

#include "SDLException.h"


GRAPHICS_BEGIN


class SDLConditional
{
public:
	class Allocator :
		public __AllocatorBase,
		public _StaticAllocatorBase<Allocator>
	{
	public:
		SDL_cond *	_new() const;
		void		_delete(SDL_cond *sdlConditional) const;
	};

protected:
	static Allocator							allocator;
	Reference<SDL_cond, void, Allocator, true>	sdlConditional;

public:
	inline SDLConditional() :	sdlConditional(&allocator)
	{	/* nothing to do */	}
	inline SDLConditional(SDLConditional &cond)	: sdlConditional(cond.sdlConditional)
	{	/* nothing to do */	}
	virtual ~SDLConditional();

	inline SDLConditional &	operator =(SDLConditional &cond)
	{
		if(&cond != this)
			sdlConditional = cond.sdlConditional;
		return *this;
	}

	void	signal();
	void	broadcast();
	void	wait();
	void	wait(uint32_t nTimeoutMs);

	inline operator SDL_cond *()
	{	return sdlConditional.getPointer();	}
	inline operator const SDL_cond *() const
	{	return sdlConditional.getPointer();	}
};


GRAPHICS_END
#endif
