#ifndef __GRAPHICS__SDL_SEMAPHORE_H__INCLUDED__
#define __GRAPHICS__SDL_SEMAPHORE_H__INCLUDED__

#include <SDL_thread.h>

#include "SDLException.h"


GRAPHICS_BEGIN


class SDLSemaphore
{
public:
	class Allocator :
		public __AllocatorBase,
		public _StaticAllocatorBase<Allocator>
	{
	public:
		SDL_sem *	_new() const;
		void		_delete(SDL_sem *sdlSemaphore) const;
	};

protected:
	static Allocator							allocator;
	Reference<SDL_sem, void, Allocator, true>	sdlSemaphore;

public:
	inline SDLSemaphore() :	sdlSemaphore(&allocator)
	{	/* nothing to do */	}
	inline SDLSemaphore(SDLSemaphore &sem) :	sdlSemaphore(sem.sdlSemaphore)
	{	/* nothing to do */	}
	virtual ~SDLSemaphore();

	void	wait();
	bool	tryWait();
	void	wait(uint32_t nTimeoutMs);
	void	post();

	inline uint32_t	getValue()
	{	return SDL_SemValue(*this);	}

	inline operator SDL_sem *()
	{	return sdlSemaphore.getPointer();	}
	inline operator const SDL_sem *() const
	{	return sdlSemaphore.getPointer();	}
};


GRAPHICS_END
#endif
