#ifndef __GRAPHICS__SHAPE_H__INCLUDED__
#define __GRAPHICS__SHAPE_H__INCLUDED__

#include <stdio.h>
#include <map>
#include <list>

#include "Vertex.h"
#include "Object3.h"

/* MilkShape constants */
#define MS_MAX_NAME             32
#define MS_MAX_PATH             256


GRAPHICS_BEGIN


struct Tri
{
	int		v[3];	// indices into vertices[]
	int		n[3];	// indices into normals[], normal per vertex
};

typedef Vector3<float> Normal;
	

class Shape: public Object3 
{
protected:
	class Index
	{
	public:
		const uint16_t	v,
						n;

		inline Index(uint16_t _v, uint16_t _n):
			v(_v),
			n(_n)
		{	/* nothing to do */	}

		inline bool operator < (const Index &i) const
		{	return v < i.v || v == i.v && n < i.n;	}
		inline bool operator == (const Index &i) const
		{	return v == i.v && n == i.n;	}
	};

	class Group
	{
	protected:
		static GLenum _convertType(uint_t _type);

	public:
		const GLenum	type;
		const uint_t	start,
						size;

		inline Group(uint_t _type, uint_t _start, uint_t _size) :
			type(_convertType(_type)),
			start(_start),
			size(_size)
		{	/* nothing to do */	}
	};

	typedef map<Index, uint16_t>	IndexMap;
	typedef map<uint16_t, Index>	VertexMap;

public:
	/*	Constructor. */
	Shape();

	/*	Destructor. */
	~Shape();

	inline void	enableTexture(const bool enable = true)
	{	bTexture = enable;	}

	//virtual bool	advance(float time);

protected:
	void	_load();
	void	_remove();

	void	_glLoad();
	void	_glLoadVertexBufferObject();
	//void	_glLoadList();

	void	_glRemove();

	void	_glRender();
	void	_glRenderVertexBufferObject();
	void	_glRenderImmediateObject();
	//void	_glRenderList();

	void	stripify(unsigned short* pIndices);
	
public:
	bool	loadFromFile( string *filename );
	bool    loadFromMs3dAsciiSegment( FILE *file );

	void	moveAndScale(const float &xdiff, const float &ydiff, const float &zdiff, const float &scale)
	{
		const uint_t	end = nIndices * 3;
		uint_t			pos = 0;

		while(pos < end)
		{
			pData[pos] = (pData[pos++] - xdiff) / scale;
			pData[pos] = (pData[pos++] - ydiff) / scale;
			pData[pos] = (pData[pos++] - zdiff) / scale;
		}
	}

	void	render( void );

	virtual void	glBind();


//private:
	unsigned int	num_vertices;
	Vertex	*vertices;

	unsigned int	num_triangles;
	Tri	*triangles;

	unsigned int	num_normals;
	Normal	*normals;

	string	*filename;
	FILE	*file;

	float	*pData;
	uint_t	nIndices,
			nID,
			nNormalsOffset,
			nTextureOffset;

	bool	bTexture;

	list<Group>	groups;
};


GRAPHICS_END
#endif
