#ifndef __GRAPHICS__SOUNDS_H__INCLUDED__
#define __GRAPHICS__SOUNDS_H__INCLUDED__

#include <fmod/fmod.h>

#include <framework/ObjectManager.h>

#include "Vector3.h"


GRAPHICS_BEGIN


// Das ist unsere Sound-Klasse.
class Sound
{
public:
	typedef	Vector3<float>	Vector;


protected:
	class Allocator :
		public __AllocatorBase,
		public _StaticAllocatorBase<Allocator>
	{
	public:
		FSOUND_SAMPLE *	_new(const string *sample) const;
		void			_delete(FSOUND_SAMPLE *sample) const;
	};


	static ObjectManager<FSOUND_SAMPLE, string, Allocator>		samples;

	string														strSample;
	ObjectManager<FSOUND_SAMPLE, string, Allocator>::Reference	sample;

	bool	m_bo_restartable,
			m_bo_loop;

	// Parameter fr den Sound.		
	int		m_int_channel;
	float	m_fl_position[3];
	int		m_int_frequency;
	int		m_int_volume;

    
public:
	Sound(const string &type, bool loop = false, bool restartable = false);
	~Sound();

	static void			initialise();
	static inline void	clean()
	{	samples.clean();	}


	// Startet einen Sound um den man sich danach nicht mehr kmmern muss.
	void	start(bool bPosition = true);
	void	stop();

	// Aktualisiert alle immerlaufende Sound.
	void update();

	// Stoppt alle immerlaufende Sound, wenn true, und startet sie wieder bei false.
	void pause(bool stop);
	
	// Setter & Getter & Adder.
	inline void	set_frequency(int frequency)
	{	m_int_frequency = frequency;	}
	inline void	set_volume(int volume)
	{	m_int_volume = volume;	}
	inline void	set_position(const Vector &position) 
	{	m_fl_position[0] = position.getX();
		m_fl_position[1] = position.getY();
		m_fl_position[2] = position.getZ(); }
	
	inline void	add_frequency(int frequency)
	{	m_int_frequency += frequency;	}
	inline void	add_volume(int volume)
	{	m_int_volume += volume;	}
	
	inline int		get_volume() const
	{	return m_int_volume;	}
	inline int		get_frequency() const
	{	return m_int_frequency;	}
	inline Vector	get_position() const
	{	return Vector(m_fl_position[0], m_fl_position[1], m_fl_position[2]);	}
};


GRAPHICS_END
#endif
