#ifndef __GRAPHICS__SPHERE_H__INCLUDED__
#define __GRAPHICS__SPHERE_H__INCLUDED__

#include "Object3.h"


GRAPHICS_BEGIN


class Sphere :	public Object3
{
public:
	class Id
	{
	public:
		const GLfloat	radius;
		const GLint		slices,
						stacks;

		inline Id(const GLfloat &_radius, const GLint &_slices, const GLint &_stacks) :
			radius(_radius),
			slices(_slices),
			stacks(_stacks)
		{	/* nothing to do */	}
	};

	typedef	StaticAllocator<Sphere, Id>				Allocator;
	typedef	Reference<Sphere, Id, Allocator, true>	Reference;


protected:
	static Allocator	allocator;

	GLint			slices,
					stacks;
	GLUquadricObj	*pSphere;


	void			_init(const GLint &_slices, const GLint &_stacks);


	virtual void	_load();
	virtual void	_remove();

	virtual void	_glLoad();
	virtual void	_glRemove();

	virtual void	_glRender();


public:
	Sphere(	const GLfloat &_radius, const GLint &_slices, const GLint &_stacks,
			const bool bCanCollide = false, Material *pMaterial = NULL			);
	Sphere(const Id &id);
	virtual ~Sphere();

	static inline Reference	create(const Id &id)
	{	Reference	ref(&allocator);
		return ref.create(&id);		}
};


GRAPHICS_END
#endif
