#ifndef __GRAPHICS__TRANSFORMATION_MATRIX_3_H__INCLUDED__
#define __GRAPHICS__TRANSFORMATION_MATRIX_3_H__INCLUDED__

#include "SquareMatrix.h"


GRAPHICS_BEGIN


template <class NumberType> class TransformationMatrix3 :
	public SquareMatrix<NumberType, 3>
{
public:
	typedef SquareMatrix<NumberType, 3>			_Base;
	typedef TransformationMatrix3<NumberType>	_Type;

	inline TransformationMatrix3<NumberType>()\
	{	/* nothing to do */	}\
	inline TransformationMatrix3<NumberType>(NumberType *pMatrix) :			_Base(pMatrix)\
	{	/* nothing to do */	}\
	inline TransformationMatrix3<NumberType>(const NumberType *pMatrix) :	_Base(pMatrix)\
	{	/* nothing to do */	}\
	inline TransformationMatrix3<NumberType>(const _TopBase &matrix) :		_Base(matrix)\
	{	/* nothing to do */	}\

	inline _Type &	operator =(NumberType *pMatrix)
	{	_Base::operator =(pMatrix);
		return *this;				}
	inline _Type &	operator =(const NumberType *pMatrix)
	{	_Base::operator =(pMatrix);
		return *this;				}
	inline _Type &	operator =(const _TopBase &matrix)
	{	_Base::operator =(matrix);
		return *this;				}

	_Type &			rotation(	const NumberType &x,
								const NumberType &y,
								const NumberType &z,
								const NumberType &theta	)
	{
		typedef NumberType	NT;

		NumberType	c = cos(theta),
					s = sin(theta),
					t = (NT)1.0 - c;

		element(0) = t*x*x + c;		_element(1) = t*x*y - s*z;	_element(2) = t*x*z + s*y;
		_element(3) = t*x*y + s*z;	_element(4) = t*y*y + c;	_element(5) = t*y*z - s*x;
		_element(6) = t*x*z - s*y;	_element(7) = t*y*z + s*x;	_element(8) = t*z*z + c;

		return *this;
	}
	static _Type	getRotation(	const NumberType &x,
									const NumberType &y,
									const NumberType &z,
									const NumberType &theta	)
	{
		_Type	matrix;
		matrix.rotation(x, y, z, theta);
		return matrix;
	}
	_Type &			rotationX(const NumberType &phi)
	{
		typedef NumberType	NT;

		NumberType	cosPhi = cos(phi),
					sinPhi = sin(phi);

		element(0) = (NT)1.0;	_element(1) = (NT)0.0;	_element(2) = (NT)0.0;
		_element(3) = (NT)0.0;	_element(4) = cosPhi;	_element(5) = -sinPhi;
		_element(6) = (NT)0.0;	_element(7) = sinPhi;	_element(8) = cosPhi;

		return *this;
	}
	static _Type	getRotationX(const NumberType &phi)
	{
		_Type	matrix;
		matrix.rotationX(phi);
		return matrix;
	}
	_Type &			rotationY(const NumberType &theta)
	{
		typedef NumberType	NT;

		NumberType	cosTheta = cos(theta),
					sinTheta = sin(theta);

		element(0) = cosTheta;		_element(1) = (NT)0.0;	_element(2) = sinTheta;
		_element(3) = (NT)0.0;		_element(4) = (NT)1.0;	_element(5) = (NT)0.0;
		_element(6) = -sinTheta;	_element(7) = (NT)0.0;	_element(8) = cosTheta;

		return *this;
	}
	static _Type	getRotationY(const NumberType &theta)
	{
		_Type	matrix;
		matrix.rotationY(theta);
		return matrix;
	}
	_Type &			rotationZ(const NumberType &psi)
	{
		typedef NumberType	NT;

		NumberType	cosPsi = cos(psi),
					sinPsi = sin(psi);

		element(0) = cosPsi;	_element(1) = -sinPsi;	_element(2) = (NT)0.0;
		_element(3) = sinPsi;	_element(4) = cosPsi;	_element(5) = (NT)0.0;
		_element(6) = (NT)0.0;	_element(7) = (NT)0.0;	_element(8) = (NT)1.0;

		return *this;
	}
	static _Type	getRotationZ(const NumberType &psi)
	{
		_Type	matrix;
		matrix.rotationZ(psi);
		return matrix;
	}

	inline _Type &	preRotate(	const NumberType &x,
								const NumberType &y,
								const NumberType &z,
								const NumberType &theta	)
	{	return *this = (*this) * getRotation(x, y, z, theta);	}
	inline _Type &	postRotate(	const NumberType &x,
								const NumberType &y,
								const NumberType &z,
								const NumberType &theta	)
	{	return *this = getRotation(x, y, z, theta) * (*this);	}

	inline _Type &	preRotateX(const NumberType &phi)
	{	return *this = (*this) * getRotationX(phi);		}
	inline _Type &	postRotateX(const NumberType &phi)
	{	return *this = getRotationX(phi) * (*this);		}
	inline _Type &	preRotateY(const NumberType &theta)
	{	return *this = (*this) * getRotationY(theta);	}
	inline _Type &	postRotateY(const NumberType &theta)
	{	return *this = getRotationY(theta) * (*this);	}
	inline _Type &	preRotateZ(const NumberType &psi)
	{	return *this = (*this) * getRotationZ(psi);		}
	inline _Type &	postRotateZ(const NumberType &psi)
	{	return *this = getRotationZ(psi) * (*this);		}
};


GRAPHICS_END
#endif
