#ifndef __GRAPHICS__TRANSFORMATION_MATRIX_4_H__INCLUDED__
#define __GRAPHICS__TRANSFORMATION_MATRIX_4_H__INCLUDED__

#include "SquareMatrix.h"


GRAPHICS_BEGIN


template <class NumberType> class TransformationMatrix4 :
	public SquareMatrix<NumberType, 4>
{
public:
	typedef SquareMatrix<NumberType, 4>			_Base;
	typedef TransformationMatrix4<NumberType>	_Type;

	inline TransformationMatrix4<NumberType>()\
	{	/* nothing to do */	}\
	inline TransformationMatrix4<NumberType>(NumberType *pMatrix) :			_Base(pMatrix)\
	{	/* nothing to do */	}\
	inline TransformationMatrix4<NumberType>(const NumberType *pMatrix) :	_Base(pMatrix)\
	{	/* nothing to do */	}\
	inline TransformationMatrix4<NumberType>(const _TopBase &matrix) :		_Base(matrix)\
	{	/* nothing to do */	}\

	inline _Type &	operator =(NumberType *pMatrix)
	{	_Base::operator =(pMatrix);
		return *this;				}
	inline _Type &	operator =(const NumberType *pMatrix)
	{	_Base::operator =(pMatrix);
		return *this;				}
	inline _Type &	operator =(const _TopBase &matrix)
	{	_Base::operator =(matrix);
		return *this;				}

	_Type &			rotation(	const NumberType &x,
								const NumberType &y,
								const NumberType &z,
								const NumberType &theta	)
	{
		typedef NumberType	NT;

		NumberType	c = cos(theta),
					s = sin(theta),
					t = (NT)1.0 - c;

		element(0) = t*x*x + c;		_element(1) = t*x*y - s*z;	_element(2) = t*x*z + s*y;	_element(3) = (NT)0.0;
		_element(4) = t*x*y + s*z;	_element(5) = t*y*y + c;	_element(6) = t*y*z - s*x;	_element(7) = (NT)0.0;
		_element(8) = t*x*z - s*y;	_element(9) = t*y*z + s*x;	_element(10) = t*z*z + c;	_element(11) = (NT)0.0;
		_element(12) = (NT)0.0;		_element(13) = (NT)0.0;		_element(14) = (NT)0.0;		_element(15) = (NT)1.0;

		return *this;
	}
	static _Type	getRotation(	const NumberType &x,
									const NumberType &y,
									const NumberType &z,
									const NumberType &theta	)
	{
		_Type	matrix;
		matrix.rotation(x, y, z, theta);
		return matrix;
	}
	_Type &			rotationX(const NumberType &phi)
	{
		typedef NumberType	NT;

		NumberType	cosPhi = cos(phi),
					sinPhi = sin(phi);

		element(0) = (NT)1.0;	_element(1) = (NT)0.0;	_element(2) = (NT)0.0;	_element(3) = (NT)0.0;
		_element(4) = (NT)0.0;	_element(5) = cosPhi;	_element(6) = -sinPhi;	_element(7) = (NT)0.0;
		_element(8) = (NT)0.0;	_element(9) = sinPhi;	_element(10) = cosPhi;	_element(11) = (NT)0.0;
		_element(12) = (NT)0.0;	_element(13) = (NT)0.0;	_element(14) = (NT)0.0;	_element(15) = (NT)1.0;

		return *this;
	}
	static _Type	getRotationX(const NumberType &phi)
	{
		_Type	matrix;
		matrix.rotationX(phi);
		return matrix;
	}
	_Type &			rotationY(const NumberType &theta)
	{
		typedef NumberType	NT;

		NumberType	cosTheta = cos(theta),
					sinTheta = sin(theta);

		element(0) = cosTheta;		_element(1) = (NT)0.0;	_element(2) = sinTheta;		_element(3) = (NT)0.0;
		_element(4) = (NT)0.0;		_element(5) = (NT)1.0;	_element(6) = (NT)0.0;		_element(7) = (NT)0.0;
		_element(8) = -sinTheta;	_element(9) = (NT)0.0;	_element(10) = cosTheta;	_element(11) = (NT)0.0;
		_element(12) = (NT)0.0;		_element(13) = (NT)0.0;	_element(14) = (NT)0.0;		_element(15) = (NT)1.0;

		return *this;
	}
	static _Type	getRotationY(const NumberType &theta)
	{
		_Type	matrix;
		matrix.rotationY(theta);
		return matrix;
	}
	_Type &			rotationZ(const NumberType &psi)
	{
		typedef NumberType	NT;

		NumberType	cosPsi = cos(psi),
					sinPsi = sin(psi);

		element(0) = cosPsi;	_element(1) = -sinPsi;	_element(2) = (NT)0.0;	_element(3) = (NT)0.0;
		_element(4) = sinPsi;	_element(5) = cosPsi;	_element(6) = (NT)0.0;	_element(7) = (NT)0.0;
		_element(8) = (NT)0.0;	_element(9) = (NT)0.0;	_element(10) = (NT)1.0;	_element(11) = (NT)0.0;
		_element(12) = (NT)0.0;	_element(13) = (NT)0.0;	_element(14) = (NT)0.0;	_element(15) = (NT)1.0;

		return *this;
	}
	static _Type	getRotationZ(const NumberType &psi)
	{
		_Type	matrix;
		matrix.rotationZ(psi);
		return matrix;
	}

	_Type &			translation(	const NumberType &x,
									const NumberType &y,
									const NumberType &z	)
	{
		typedef NumberType	NT;

		element(0) = (NT)1.0;	_element(1) = (NT)0.0;	_element(2) = (NT)0.0;	_element(3) = x;
		_element(4) = (NT)0.0;	_element(5) = (NT)1.0;	_element(6) = (NT)0.0;	_element(7) = y;
		_element(8) = (NT)0.0;	_element(9) = (NT)0.0;	_element(10) = (NT)1.0;	_element(11) = z;
		_element(12) = (NT)0.0;	_element(13) = (NT)0.0;	_element(14) = (NT)0.0;	_element(15) = (NT)1.0;

		return *this;
	}
	static _Type	getTranslation(	const NumberType &x,
									const NumberType &y,
									const NumberType &z	)
	{
		_Type	matrix;
		matrix.translation(x, y, z);
		return matrix;
	}

	_Type &			scale(	const NumberType &sx,
							const NumberType &sy,
							const NumberType &sz	)
	{
		typedef NumberType	NT;

		element(0) = sx;		_element(1) = (NT)0.0;	_element(2) = (NT)0.0;	_element(3) = (NT)0.0;
		_element(4) = (NT)0.0;	_element(5) = sy;		_element(6) = (NT)0.0;	_element(7) = (NT)0.0;
		_element(8) = (NT)0.0;	_element(9) = (NT)0.0;	_element(10) = sz;		_element(11) = (NT)0.0;
		_element(12) = (NT)0.0;	_element(13) = (NT)0.0;	_element(14) = (NT)0.0;	_element(15) = (NT)1.0;

		return *this;
	}
	static _Type	getScale(	const NumberType &sx,
								const NumberType &sy,
								const NumberType &sz	)
	{
		_Type	matrix;
		matrix.scale(sx, sy, sz);
		return matrix;
	}

	inline _Type &	preRotate(	const NumberType &x,
								const NumberType &y,
								const NumberType &z,
								const NumberType &theta	)
	{	return *this = (*this) * getRotation(x, y, z, theta);	}
	inline _Type &	postRotate(	const NumberType &x,
								const NumberType &y,
								const NumberType &z,
								const NumberType &theta	)
	{	return *this = getRotation(x, y, z, theta) * (*this);	}

	inline _Type &	preRotateX(const NumberType &phi)
	{	return *this = (*this) * getRotationX(phi);		}
	inline _Type &	postRotateX(const NumberType &phi)
	{	return *this = getRotationX(phi) * (*this);		}
	inline _Type &	preRotateY(const NumberType &theta)
	{	return *this = (*this) * getRotationY(theta);	}
	inline _Type &	postRotateY(const NumberType &theta)
	{	return *this = getRotationY(theta) * (*this);	}
	inline _Type &	preRotateZ(const NumberType &psi)
	{	return *this = (*this) * getRotationZ(psi);		}
	inline _Type &	postRotateZ(const NumberType &psi)
	{	return *this = getRotationZ(psi) * (*this);		}

	_Type &			preTranslate(	const NumberType &x,
									const NumberType &y,
									const NumberType &z	)
	{
		element(0, 3) +=	getElement(0, 0) * x +	getElement(0, 1) * y +	getElement(0, 2) * z;
		_element(1, 3) +=	getElement(1, 0) * x +	getElement(1, 1) * y +	getElement(1, 2) * z;
		_element(2, 3) +=	getElement(2, 0) * x +	getElement(2, 1) * y +	getElement(2, 2) * z;

		return *this;
	}
	_Type &			postTranslate(	const NumberType &x,
									const NumberType &y,
									const NumberType &z	)
	{
		element(0, 3) += x;
		_element(1, 3) += y;
		_element(2, 3) += z;
		return *this;
	}

	_Type &			preScale(	const NumberType &sx,
								const NumberType &sy,
								const NumberType &sz	)
	{
		element(0, 0) *= sx;
		_element(1, 0) *= sx;
		_element(2, 0) *= sx;
		_element(0, 1) *= sy;
		_element(1, 1) *= sy;
		_element(2, 1) *= sy;
		_element(0, 2) *= sz;
		_element(1, 2) *= sz;
		_element(2, 2) *= sz;

		return *this;
	}
	_Type &			postScale(	const NumberType &sx,
								const NumberType &sy,
								const NumberType &sz	)
	{
		element(0, 0) *= sx;
		_element(0, 1) *= sx;
		_element(0, 2) *= sx;
		_element(0, 3) *= sx;
		_element(1, 0) *= sy;
		_element(1, 1) *= sy;
		_element(1, 2) *= sy;
		_element(1, 3) *= sy;
		_element(2, 0) *= sz;
		_element(2, 1) *= sz;
		_element(2, 2) *= sz;
		_element(2, 3) *= sz;

		return *this;
	}
};


GRAPHICS_END
#endif
