#ifndef __GRAPHICS__VECTOR_2_H__INCLUDED__
#define __GRAPHICS__VECTOR_2_H__INCLUDED__

#include <math.h>

#include "Matrix.h"


GRAPHICS_BEGIN


template <class NumberType> class Vector2 :	public Matrix<NumberType, 2, 1>
{
public:
	typedef Matrix<NumberType, 2, 1>	_Base;
	typedef Vector2<NumberType>			_Type;

	inline Vector2<NumberType>()
	{	/* nothing to do */	}
	inline Vector2<NumberType>(NumberType *pVector) :		_Base(pVector)
	{	/* nothing to do */	}
	inline Vector2<NumberType>(const NumberType *pVector) :	_Base(pVector)
	{	/* nothing to do */	}
	inline Vector2<NumberType>(const _TopBase &vector) :	_Base(vector)
	{	/* nothing to do */	}
	inline Vector2<NumberType>(const NumberType &_x, const NumberType &_y)
	{	_element(0) = _x;
		_element(1) = _y;	}

	inline _Type &	operator =(NumberType *pVector)
	{	_Base::operator =(pVector);
		return *this;				}
	inline _Type &	operator =(const NumberType *pVector)
	{	_Base::operator =(pVector);
		return *this;				}
	inline _Type &	operator =(const _TopBase &vector)
	{	_Base::operator =(vector);
		return *this;				}

	inline NumberType &			x()
	{	return element(0);	}
	inline const NumberType &	getX() const
	{	return getElement(0);	}
	inline NumberType &			y()
	{	return element(1);	}
	inline const NumberType &	getY() const
	{	return getElement(1);	}

	inline void	set(const NumberType &_x, const NumberType &_y)
	{	_element(0) = _x;
		_element(1) = _y;	}

	inline NumberType			magnitudeSquared() const
	{	return getX() * getX() + getY() * getY();	}
	inline NumberType			magnitude() const
	{	return sqrt(magnitudeSquared());	}

	inline _Type &				normalise()
	{	operator /=(magnitude());
		return *this;			}
	void						getNormalised(_Type &normalised) const
	{
		NumberType	m = magnitude();

		normalised.element(0) = getX() / m;
		normalised._element(1) = getY() / m;
	}
	_Type						getNormalised() const
	{
		_Type	normalised;
		getNormalised(normalised);
		return normalised;
	}

	inline NumberType &			operator[](uint_t index)
	{	return element(index);	}
	inline const NumberType &	operator[](uint_t index) const
	{	return getElement(index);	}
};


GRAPHICS_END
#endif
