#ifndef __GRAPHICS__VECTOR_3_H__INCLUDED__
#define __GRAPHICS__VECTOR_3_H__INCLUDED__

#include <math.h>

#include "Matrix.h"


GRAPHICS_BEGIN


template <class NumberType> class Vector3 :	public Matrix<NumberType, 3, 1>
{
public:
	typedef Matrix<NumberType, 3, 1>	_Base;
	typedef Vector3<NumberType>			_Type;

	inline Vector3<NumberType>()
	{	/* nothing to do */	}
	inline Vector3<NumberType>(NumberType *pVector) :		_Base(pVector)
	{	/* nothing to do */	}
	inline Vector3<NumberType>(const NumberType *pVector) :	_Base(pVector)
	{	/* nothing to do */	}
	inline Vector3<NumberType>(const _TopBase &vector) :	_Base(vector)
	{	/* nothing to do */	}
	Vector3<NumberType>(const Matrix<NumberType, 4, 1> &vector)
	{
		_element(0) = vector.getElement(0, 0);
		_element(1) = vector.getElement(1, 0);
		_element(2) = vector.getElement(2, 0);
	}
	Vector3<NumberType>(	const NumberType &_x,
							const NumberType &_y,
							const NumberType &_z	)
	{
		_element(0) = _x;
		_element(1) = _y;
		_element(2) = _z;
	}

	inline _Type &	operator =(NumberType *pVector)
	{	_Base::operator =(pVector);
		return *this;				}
	inline _Type &	operator =(const NumberType *pVector)
	{	_Base::operator =(pVector);
		return *this;				}
	inline _Type &	operator =(const _TopBase &vector)
	{	_Base::operator =(vector);
		return *this;				}
	inline _Type &	operator =(const Matrix<NumberType, 4, 1> &vector)
	{
		element(0) = vector.getElement(0, 0);
		_element(1) = vector.getElement(1, 0);
		_element(2) = vector.getElement(2, 0);
		return *this;
	}

	inline NumberType &			x()
	{	return element(0);	}
	inline const NumberType &	getX() const
	{	return getElement(0);	}
	inline NumberType &			y()
	{	return element(1);	}
	inline const NumberType &	getY() const
	{	return getElement(1);	}
	inline NumberType &			z()
	{	return element(2);	}
	inline const NumberType &	getZ() const
	{	return getElement(2);	}

	void set(	const NumberType &_x,
				const NumberType &_y,
				const NumberType &_z	)
	{
		element(0) = _x;
		_element(1) = _y;
		_element(2) = _z;
	}

	inline NumberType			magnitudeSquared() const
	{	return getX() * getX() + getY() * getY() + getZ() * getZ();	}
	inline NumberType			magnitude() const
	{	return sqrt(magnitudeSquared());	}

	inline _Type &				normalise()
	{	operator /=(magnitude());
		return *this;			}
	void						getNormalised(_Type &normalised) const
	{
		NumberType	m = magnitude();

		normalised.element(0) = getX() / m;
		normalised._element(1) = getY() / m;
		normalised._element(2) = getZ() / m;
	}
	_Type						getNormalised() const
	{
		_Type	normalised;
		getNormalised(normalised);
		return normalised;
	}

	inline _Type &				cross(const _Type &vector)
	{	operator =(getCrossed(vector));
		return *this;					}
	void						getCrossed(_Type &crossed, const _Type &vector) const
	{
		crossed.element(0) = getY() * vector.getZ() - vector.getY() * getZ();
		crossed._element(1) = getZ() * vector.getX() - vector.getZ() * getX();
		crossed._element(2) = getX() * vector.getY() - vector.getX() * getY();
	}
	inline _Type				getCrossed(const _Type &vector) const
	{
		_Type	crossed;
		getCrossed(crossed, vector);
		return crossed;
	}

	NumberType					dot(const _Type &vector) const
	{
		return	getElement(0) * vector.getElement(0) +
				getElement(1) * vector.getElement(1) +
				getElement(2) * vector.getElement(2);
	}

	inline _Type &				multiplyLeft(const Matrix<NumberType, 3, 3> &matrix)
	{	*this = matrix * (*this);
		return *this;				}
	inline _Type &				multiplyLeft(const Matrix<NumberType, 4, 4> &matrix)
	{	*this = matrix * (*this);
		return *this;				}

	inline NumberType &			operator[](uint_t index)
	{	return element(index);	}
	inline const NumberType &	operator[](uint_t index) const
	{	return getElement(index);	}
};

template <class NumberType> Vector3<NumberType>			operator *(const Matrix<NumberType, 4, 4> &matrix, const Vector3<NumberType> &vector)
{
	return Vector3<NumberType>(	vector.getX() * matrix.getElement(0, 0) + vector.getY() * matrix.getElement(0, 1) + vector.getZ() * matrix.getElement(0, 2),
								vector.getX() * matrix.getElement(1, 0) + vector.getY() * matrix.getElement(1, 1) + vector.getZ() * matrix.getElement(1, 2),
								vector.getX() * matrix.getElement(2, 0) + vector.getY() * matrix.getElement(2, 1) + vector.getZ() * matrix.getElement(2, 2)	);
}


GRAPHICS_END
#endif
