#ifndef __GRAPHICS__VERTEX_H__INCLUDED__
#define __GRAPHICS__VERTEX_H__INCLUDED__

#include "Vector4.h"

GRAPHICS_BEGIN

class Vertex: public Vector4<float>
{
public:
	typedef Vector4<float>			_Base;
	typedef Vertex					_Type;


	float	u,v;	// texture position
	int		bone;	// index into Model bones array


	inline Vertex()
	{	/* nothing to do */	}
	inline Vertex(float *pVector) :		_Base(pVector)
	{	/* nothing to do */	}
	inline Vertex(const float *pVector) :	_Base(pVector)
	{	/* nothing to do */	}
	inline Vertex(const _TopBase &vector) :	_Base(vector)
	{	/* nothing to do */	}
	inline Vertex(	const float &_x,
					const float &_y,
					const float &_z,
					const float &_w = 1.0f	) :	_Base(_x,_y,_z, _w)
	{	/* nothing to do */	}

	virtual ~Vertex();

	inline void inverseRotateVect(const Matrix<float, 4, 4> &matrix)
	{
		float	x = getX() * matrix.getElement(0, 0) + getY() * matrix.getElement(1, 0) + getZ() * matrix.getElement(2, 0),
				y = getX() * matrix.getElement(0, 1) + getY() * matrix.getElement(1, 1) + getZ() * matrix.getElement(2, 1),
				z = getX() * matrix.getElement(0, 2) + getY() * matrix.getElement(1, 2) + getZ() * matrix.getElement(2, 2);

		set(x, y, z);
	}

	inline void inverseTranslateVect(const Matrix<float, 4, 4> &matrix)
	{
		element(0) -= matrix.getElement(0, 3);
		_element(1) -= matrix.getElement(1, 3);
		_element(2) -= matrix.getElement(2, 3);
	}


	inline _Type &	operator =(float *pVector)
	{	_Base::operator =(pVector);
		return *this;				}
	inline _Type &	operator =(const float *pVector)
	{	_Base::operator =(pVector);
		return *this;				}
	inline _Type &	operator =(const _TopBase &vector)
	{	_Base::operator =(vector);
		return *this;				}
};

GRAPHICS_END

#endif