#include <aquawars/Projectile.h>

#include <graphics/Map.h>
#include <graphics/ParticleList.h>
#include <graphics/Scene.h>

#include <aquawars/ObjectTypes.h>
#include <aquawars/Vehicle.h>


Sound	Projectile::sndBullet0("res/sounds/impact0.wav"),
		Projectile::sndBullet1("res/sounds/impact1.wav"),
		Projectile::sndBullet2("res/sounds/impact2.wav"),
		Projectile::sndBullet3("res/sounds/impact3.wav"),
		Projectile::sndBullet4("res/sounds/impact4.wav"),
		Projectile::sndBullet5("res/sounds/impact5.wav"),
		Projectile::sndBullet6("res/sounds/impact6.wav"),
		Projectile::sndBullet7("res/sounds/impact7.wav"),
		Projectile::sndBullet8("res/sounds/impact8.wav"),
		Projectile::sndBullet9("res/sounds/impact9.wav"),
		Projectile::sndExplosion0("res/sounds/explosion0.wav"),
		Projectile::sndExplosion1("res/sounds/explosion1.wav"),
		Projectile::sndExplosion2("res/sounds/explosion2.wav"),
		Projectile::sndExplosion3("res/sounds/explosion3.wav"),
		Projectile::sndExplosion4("res/sounds/explosion4.wav"),
		Projectile::sndExplosion5("res/sounds/explosion5.wav"),
		Projectile::sndExplosion6("res/sounds/explosion6.wav"),
		Projectile::sndExplosion7("res/sounds/explosion7.wav");

vector<Sound *>	Bullet::impactSounds,
				GaussBullet::impactSounds,
				Torpedo::impactSounds;


void Projectile::addSounds()
{
	Bullet::addSound(&sndBullet0);
	Bullet::addSound(&sndBullet1);
	Bullet::addSound(&sndBullet2);
	Bullet::addSound(&sndBullet3);
	Bullet::addSound(&sndBullet4);
	Bullet::addSound(&sndBullet5);
	Bullet::addSound(&sndBullet6);
	Bullet::addSound(&sndBullet7);
	Bullet::addSound(&sndBullet8);
	Bullet::addSound(&sndBullet9);

	GaussBullet::addSound(&sndExplosion0);
	GaussBullet::addSound(&sndExplosion1);
	GaussBullet::addSound(&sndExplosion2);
	GaussBullet::addSound(&sndExplosion3);
	GaussBullet::addSound(&sndExplosion4);
	GaussBullet::addSound(&sndExplosion5);
	GaussBullet::addSound(&sndExplosion6);
	GaussBullet::addSound(&sndExplosion7);

	Torpedo::addSound(&sndExplosion0);
	Torpedo::addSound(&sndExplosion1);
	Torpedo::addSound(&sndExplosion2);
	Torpedo::addSound(&sndExplosion3);
	Torpedo::addSound(&sndExplosion4);
	Torpedo::addSound(&sndExplosion5);
	Torpedo::addSound(&sndExplosion6);
	Torpedo::addSound(&sndExplosion7);
}


bool Projectile::advance(float time)
{
	fLifetime -= time;

	if(fLifetime <= 0.0f)
		return false;

	postTranslate(getForward() * fSpeed * time);
	updateFinal();

	list<_Object3 *>	objects;

	pScene->getCollisions(objects, this);
	for(list<_Object3 *>::iterator it = objects.begin(); it != objects.end(); it++)
		switch((*it)->getCathegoryID())
		{
		case c_terrain:
			{
				switch((*it)->getTypeID())
				{
				case wallx:
				case wallz:
					// projektil trifft wand - verschwinden lassen
					return false;

				case terrain:
					{
						// projektil ist in einem map-block - hhe abfragen und mit position vergleichen
						Map		*pMap = (Map *)((*it)->getTypeData());
						Vector	pos(getPosition());

						if(pMap->heightAt(pos.getX(), pos.getZ()) > pos.getY())
						{
							impact(pos, true);							
							return false;
						}
						break;
					}
				}
				break;
			}

		case c_vehicle:
			{
				if(*it == parent)
					break;

				Vehicle	*pVehicle = (Vehicle *)(*it);
				if(!pVehicle->isDestroyed())
				{
					Vector	pos(getPosition());

					pVehicle->onHit(pos, fDamage);
					if(pVehicle->isDestroyed())
						parent->onTargetDestroyed(pVehicle);

					impact(pos);
				}

				return false;
			}
		}

	return true;
}


void Bullet::impact(const Vector &position, bool terrain)
{
	// create an impact particle system
	if(terrain)
	{
		spawn(ParticleList::createExplosion3D(0.0f, 0.0f, 1.0f, 10, 0, position));
		spawn(ParticleList::createCloudExplosion(1, getPosition(), 2.0f));
	}
	else
		spawn(ParticleList::createExplosion3D(0.0f, 0.0f, 1.0f, 10, 40, position));

	// play the impact sound
	if(impactSounds.size())
	{
		Sound *sound = impactSounds[rand() % impactSounds.size()];

		sound->set_position(position);
		sound->start();
	}
}


void GaussBullet::impact(const Vector &position, bool terrain)
{
	// create an impact particle system
	if(terrain)
	{
		spawn(ParticleList::createExplosion3D(0.01f, 0.0f, 1.0f, 10, 0, position));
		spawn(ParticleList::createCloudExplosion(3, getPosition(), 2.0f));
	}
	else
		spawn(ParticleList::createExplosion3D(0.01f, 0.0f, 1.0f, 10, 50, position));

	// play the impact sound
	if(impactSounds.size())
	{
		Sound *sound = impactSounds[rand() % impactSounds.size()];

		sound->set_position(position);
		sound->start();
	}
}


void Torpedo::impact(const Vector &position, bool terrain)
{
	// create an impact particle system
	if(terrain)
	{
		spawn(ParticleList::createExplosion3D(0.1f, 0.1f, 2.5f, 10, 0, position));
		spawn(ParticleList::createCloudExplosion(15, getPosition(), 2.5f));
	}
	else
		spawn(ParticleList::createExplosion3D(0.1f, 0.1f, 2.5f, 10, 75, position));


	// play the impact sound
	if(impactSounds.size())
	{
		Sound *sound = impactSounds[rand() % impactSounds.size()];

		sound->set_position(position);
		sound->start();
	}
}
