#ifdef _WINDOWS
#include <framework/_windows.h>
#else
#include <errno.h>
#endif

#include <framework/Exception.h>


FRAMEWORK_BEGIN

using namespace std;



Error &	Error::operator =(const Error &e)
{
	if(&e != this)
		exception::operator =(e);
	return *this;
}


void Exception::init()
{
	str = strException;
	if(!strCause.empty())
		str += string(" (") + strCause + ")";
	else if(!refCause.isEmpty())
	{	strCause = "caused by: " + refCause->str;
		str += string("\n\t") + strCause;			}
}

Exception::Exception(const string &_strCause, const string &_strException) :
	exception((_strException + (_strCause.empty() ? string() : (string("(") + _strCause + ")"))).c_str()),
	strException(_strException),
	strCause(_strCause)
{	init();	}
Exception::Exception(const Exception &eCause, const string &_strException) :
	exception(strException.c_str()),
	strException(_strException),
	refCause(eCause)
{	init();	}

Exception & Exception::operator =(const Exception &e)
{
	if(&e != this)
	{
		exception::operator =(e);
		str = e.str;
		strException = e.strException;
		strCause = e.strCause;
		refCause = e.refCause;
	}
	return *this;
}


void Exception::report() const
{
#if defined(_WINDOWS)
		MessageBox(NULL, strCause.c_str(), strException.c_str(), MB_ICONERROR);
#else
		cerr << strException << ": " << strCause << endl;
#endif
}

const char * Exception::what() const
{	return str.c_str();	}


#ifdef _WINDOWS
string	getLastErrorString()
{
	string	strError;
	uint_t	nLastError = GetLastError();
	char *	pErrorBuffer;

	if(!FormatMessage(	FORMAT_MESSAGE_ALLOCATE_BUFFER | 
						FORMAT_MESSAGE_FROM_SYSTEM | 
						FORMAT_MESSAGE_IGNORE_INSERTS,
						NULL,
						nLastError,
						MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
						(LPTSTR)&pErrorBuffer,
						0,
						NULL										)	)
	{	char buffer[128];
		strError = sprintf(buffer, "unknown error 0x%08x\n", nLastError);	}
	else
		strError = pErrorBuffer;

	LocalFree(pErrorBuffer);
	return strError;
}
#endif


FRAMEWORK_END
