#include <framework/RLEInputBuffer.h>


FRAMEWORK_BEGIN


void RLEInputBuffer::_read(uint8_t *pEl)
{
	uint8_t	packet;

	if(nRemaining <= 0)
	{
		// read first packet byte (type and length)
		_Base::_read(&packet);

		// compressed?
		bCompressed = packet & 0x80 ? true : false;
		// get the length
		nRemaining = ((packet & 0x7f) + 1) * nWordLength;

		if(bCompressed)
		{
			// load word
			_Base::_read(word, nWordLength);
			nWordIndex = 0;
		}
	}

	if(bCompressed)
	{
		*pEl = word[nWordIndex++];
		if(nWordIndex >= nWordLength)
			nWordIndex -= nWordLength;
	}
	else
		_Base::_read(pEl);
	nRemaining--;
}

int_t RLEInputBuffer::_read(uint8_t *_buffer, int_t length)
{
	uint_t	nRead,
			nCopied = 0;
	uint8_t	packet;

	if(nRemaining)
	{
		nRead = nRemaining < length ? nRemaining : length;
		if(bCompressed)
		{
			// read compressed
			for(uint_t i = 0; i < nRead; i++)
			{
				*(_buffer++) = word[nWordIndex++];
				if(nWordIndex >= nWordLength)
					nWordIndex = 0;
			}
		}
		else
		{
			// read uncompressed
			nRead = _Base::_read(_buffer, nRead);

			_buffer += nRead;
			nWordIndex = (nWordIndex + nRead) % nWordLength;
		}

		length -= nRead;
		nRemaining -= nRead;
		nCopied += nRead;
	}

	while((length > 0) && !bEOF)
	{
		// read first packet byte (type and length)
		_Base::_read(&packet);

		// compressed?
		bCompressed = packet & 0x80 ? true : false;
		// get the length
		nRemaining = ((packet & 0x7f) + 1) * nWordLength;

		nRead = nRemaining < length ? nRemaining : length;
		if(bCompressed)
		{
			// read compressed
			_Base::_read(word, nWordLength);
			if(bEOF)
				throw EndOfFileException();

			for(uint_t i = 0; i < nRead; i++)
			{
				*(_buffer++) = word[nWordIndex++];
				if(nWordIndex >= nWordLength)
					nWordIndex = 0;
			}
		}
		else
		{
			// read uncompressed
			nRead = _Base::_read(_buffer, nRead);

			_buffer += nRead;
			nWordIndex = (nWordIndex + nRead) % nWordLength;
		}

		length -= nRead;
		nRemaining -= nRead;
		nCopied += nRead;
	}

	if(!nCopied)
		throw EndOfFileException();
	return nCopied;
}

int_t RLEInputBuffer::_skip(int_t length)
{
	uint_t	nSkip,
			nSkipped = 0;
	uint8_t	packet;

	if(nRemaining)
	{
		nSkip = nRemaining < length ? nRemaining : length;
		if(!bCompressed)
		{
			// read uncompressed
			nSkip = _Base::_skip(nSkip);
			nWordIndex = (nWordIndex + nSkip) % nWordLength;
		}

		length -= nSkip;
		nRemaining -= nSkip;
		nSkipped += nSkip;
	}

	while((length > 0) && !bEOF)
	{
		// read first packet byte (type and length)
		_Base::_read(&packet);

		// compressed?
		bCompressed = packet & 0x80 ? true : false;
		// get the length
		nRemaining = ((packet & 0x7f) + 1) * nWordLength;

		nSkip = nRemaining < length ? nRemaining : length;
		if(bCompressed)
		{
			// read compressed
			_Base::_skip(nWordLength);
			if(bEOF)
				throw EndOfFileException();
		}
		else
		{
			// read uncompressed
			nSkip = _Base::_skip(nSkip);
			nWordIndex = (nWordIndex + nSkip) % nWordLength;
		}

		length -= nSkip;
		nRemaining -= nSkip;
		nSkipped += nSkip;
	}

	if(!nSkipped)
		throw EndOfFileException();
	return nSkipped;
}


RLEInputBuffer::RLEInputBuffer(	const char *szFilename,
								const uint8_t _nWordLength	) :	_Base(szFilename),
																nWordLength(_nWordLength),
																nWordIndex(0),
																nRemaining(0),
																bCompressed(false)
{
	word = new uint8_t[nWordLength];
	if(!word)
		throw OutOfMemoryError();
}

RLEInputBuffer::RLEInputBuffer(	const string &strFilename,
								const uint8_t _nWordLength	) :	_Base(strFilename),
																nWordLength(_nWordLength),
																nWordIndex(0),
																nRemaining(0),
																bCompressed(false)
{
	word = new uint8_t[nWordLength];
	if(!word)
		throw OutOfMemoryError();
}

RLEInputBuffer::~RLEInputBuffer()
{	delete word;	}


FRAMEWORK_END
