#include <fmod/fmod.h>

#include <graphics/Camera.h>


GRAPHICS_BEGIN


void Camera::_getPlanes()
{
	GLfloat					mProjection[16];

	glGetFloatv(GL_MODELVIEW_MATRIX, mProjection);

	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glMultMatrixf(mProjection);

	glGetFloatv(GL_PROJECTION_MATRIX, mProjection);

	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);


	// left clipping plane
	vPlanes[0].set(	mProjection[3 + 4 * 0] + mProjection[0 + 4 * 0],
					mProjection[3 + 4 * 1] + mProjection[0 + 4 * 1],
					mProjection[3 + 4 * 2] + mProjection[0 + 4 * 2],
					mProjection[3 + 4 * 3] + mProjection[0 + 4 * 3]	);
	vPlanes[0].normalise3();

	// right clipping plane
	vPlanes[1].set(	mProjection[3 + 4 * 0] - mProjection[0 + 4 * 0],
					mProjection[3 + 4 * 1] - mProjection[0 + 4 * 1],
					mProjection[3 + 4 * 2] - mProjection[0 + 4 * 2],
					mProjection[3 + 4 * 3] - mProjection[0 + 4 * 3]	);
	vPlanes[1].normalise3();

	// bottom clipping plane
	vPlanes[2].set(	mProjection[3 + 4 * 0] - mProjection[1 + 4 * 0],
					mProjection[3 + 4 * 1] - mProjection[1 + 4 * 1],
					mProjection[3 + 4 * 2] - mProjection[1 + 4 * 2],
					mProjection[3 + 4 * 3] - mProjection[1 + 4 * 3]	);
	vPlanes[2].normalise3();

	// top clipping plane
	vPlanes[3].set(	mProjection[3 + 4 * 0] + mProjection[1 + 4 * 0],
					mProjection[3 + 4 * 1] + mProjection[1 + 4 * 1],
					mProjection[3 + 4 * 2] + mProjection[1 + 4 * 2],
					mProjection[3 + 4 * 3] + mProjection[1 + 4 * 3]	);
	vPlanes[3].normalise3();

	// near clipping plane
	vPlanes[4].set(	mProjection[3 + 4 * 0] + mProjection[2 + 4 * 0],
					mProjection[3 + 4 * 1] + mProjection[2 + 4 * 1],
					mProjection[3 + 4 * 2] + mProjection[2 + 4 * 2],
					mProjection[3 + 4 * 3] + mProjection[2 + 4 * 3]	);
	vPlanes[4].normalise3();

	// far clipping plane
	vPlanes[5].set(	mProjection[3 + 4 * 0] - mProjection[2 + 4 * 0],
					mProjection[3 + 4 * 1] - mProjection[2 + 4 * 1],
					mProjection[3 + 4 * 2] - mProjection[2 + 4 * 2],
					mProjection[3 + 4 * 3] - mProjection[2 + 4 * 3]	);
	vPlanes[5].normalise3();
}


Camera::Camera() :	nearPlane(0.1),
					farPlane(10000.0),
					fov(45.0)
{
	distance(3.0f);
	rotate(0.0f, 0.0f);
	roll(0.0f);
}
Camera::~Camera()
{	/* nothing to do */	}


void Camera::planes(const GLdouble &_nearPlane, const GLdouble &_farPlane)
{
	nearPlane = _nearPlane < 0.000001f ? 0.000001f : _nearPlane;
	farPlane = _farPlane < (nearPlane + 0.001f) ? nearPlane + 0.001f : _farPlane;
}


void Camera::rotate(const GLdouble &_phi, const GLdouble &_theta)
{
	phi = _phi - PIx2 * floor(_phi / PIx2 + 0.5);
	theta = _theta - PIx2 * floor(_theta / PIx2 + 0.5);
}

Camera::Vector Camera::getDirection()
{
	mUpdate.P();
	Vector result(vDirection);
	mUpdate.V();

	return result;
}
Camera::VectorF Camera::getDirectionF()
{
	mUpdate.P();
	VectorF result(	(GLfloat)vDirection.getX(),
					(GLfloat)vDirection.getY(),
					(GLfloat)vDirection.getZ()	);
	mUpdate.V();

	return result;
}

Camera::Vector Camera::getPosition()
{
	mUpdate.P();
	Vector	result(vDirection * (-d));
	mUpdate.V();

	if(pTarget)
		return pTarget->getGlPositionD() + result;
	else
		return result;
}
Camera::VectorF Camera::getPositionF()
{
	mUpdate.P();
	VectorF result(	(GLfloat)(vDirection.getX() * (-d)),
					(GLfloat)(vDirection.getY() * (-d)),
					(GLfloat)(vDirection.getZ() * (-d))	);
	mUpdate.V();

	if(pTarget)
		return pTarget->getGlPosition() + result;
	else
		return result;
}


void Camera::roll(const GLdouble &_psi)
{	psi = _psi - PIx2 * floor(_psi / PIx2 + 0.5);	}

void Camera::fieldOfView(const GLdouble &_fov)
{	fov = _fov - PIx2 * floor(_fov / PIx2);			}


Camera::Visibility Camera::visibility(_Object3 *pObject)
{
	if(pObject->useVisibleRange())
	{
		VectorF		vDistance(pObject->getGlPosition() - getPositionF());

		if(vDistance.magnitudeSquared() > pObject->getVisibleRangeSquared())
			return none;
	}


	if(pObject->getBoundaryType() == _Object3::none)
		return partial;
	else if(pObject->getBoundaryType() == _Object3::sphere)
	{
		const GLfloat	*pBoundary = pObject->getBoundary(),
						fRadius = pBoundary[1];
		GLfloat			fDistance;
		VectorF			vPosition(	pObject->getGlPositionX() + pBoundary[2],
									pObject->getGlPositionY() + pBoundary[3],
									pObject->getGlPositionZ() + pBoundary[4]	);

		// calculate our distances to each of the planes
		for(int i = 0; i < 6; i++)
		{
			// find the distance to this plane
			fDistance = vPlanes[i].dot(vPosition);

			// if this distance is < -sphere.radius, we are outside
			if(fDistance < -fRadius)
				return none;

			// else if the distance is between +- radius, then we intersect
			if(fabs(fDistance) < fRadius)
				return partial;
		}

		// otherwise we are fully in view
		return full;
	}
	else if(pObject->getBoundaryType() == _Object3::box)
	{
		VectorF	box[8];
		uint_t	corners = 0,
				total = 1;

		pObject->getBoundingBox(box);

		for(uint_t j = 0; j < 6; j++)
		{
			// check if all corners are behind one plane
			corners = 8;
			for(uint_t i = 0; i < 8; i++)
				if(vPlanes[j].dot(box[i]) < 0.0f)
				{
					corners--;
					total = 0;
				}

			if(!corners)
				// all were outside - nothing is visible
				return none;
		}

		if(total)
			// all corners are inside
			return full;

		// some in, some out
		return partial;
	}
	else
		throw NotImplementedException("unknown bounding object type: " + pObject->getBoundaryType());
}


void Camera::glLook()
{
	GLfloat	viewport[4];
	glGetFloatv(GL_VIEWPORT, viewport);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(fov, viewport[2] / viewport[3], nearPlane, farPlane);
	glMatrixMode(GL_MODELVIEW);

	TransformationMatrix	transform;
	Vector					vCamera,
							vTarget,
							vForward,
							vUp,
							vStrafe;

	// compute new direction and up vectors
	if(pTarget)
	{
		vTarget = pTarget->getGlPositionD();
		vForward = pTarget->getGlForwardD();
		vUp = pTarget->getGlUpD();
	}
	else
	{
		vTarget.set(0.0, 0.0, 0.0);
		vForward.set(0.0, 0.0, -1.0);
		vUp.set(0.0, 1.0, 0.0);
	}
	vStrafe = vForward.getCrossed(vUp);

	transform.rotation(vForward.getX(), vForward.getY(), vForward.getZ(), psi);
	transform.preRotate(vUp.getX(), vUp.getY(), vUp.getZ(), theta);
	transform.preRotate(vStrafe.getX(), vStrafe.getY(), vStrafe.getZ(), phi);

	vUp = transform * vUp;

	// the direction vector has to be locked
	mUpdate.P();
	vDirection = transform * vForward;
	mUpdate.V();

	vCamera = vTarget - vDirection * d;
	vTarget += vUp * (d / 6.0f);
	glLoadIdentity();
	gluLookAt(	vCamera.getX(), vCamera.getY(), vCamera.getZ(),
				vTarget.getX(), vTarget.getY(), vTarget.getZ(),
				vUp.getX(), vUp.getY(), vUp.getZ()	);

	// tell FMOD the camera position
	float	position[3] = { (float)vCamera.getX(), (float)vCamera.getY(), (float)vCamera.getZ() };
	FSOUND_3D_Listener_SetAttributes(	position, NULL,
										-(float)vDirection.getX(), -(float)vDirection.getY(), -(float)vDirection.getZ(),
										(float)vUp.getX(), (float)vUp.getY(), (float)vUp.getZ()							);


	_getPlanes();
}


GRAPHICS_END
