#include <math.h>

#include <graphics/Location.h>


GRAPHICS_BEGIN


void Location::__transform()
{
	mUpdate.P();
	vForward = mTransform * Vector(0.0f, 0.0f, -1.0f);
	vUp = mTransform * Vector(0.0f, 1.0f, 0.0f);
	mUpdate.V();

	vForward.normalise();
	vUp.normalise();
}


void Location::reset()
{
	mUpdate.P();
	mTransform.unity();
	mUpdate.V();

	vForward.set(0.0f, 0.0f, -1.0f);
	vUp.set(0.0f, 1.0f, 0.0f);

	bTransform = false;
}


void Location::preRotate(const GLfloat &x, const GLfloat &y, const GLfloat &z, GLfloat theta)
{
	theta += (GLfloat)(PIx2 * floor(theta / PIx2));

	mUpdate.P();
	mTransform.preRotate(x, y, z, theta);
	mUpdate.V();

	bTransform = true;
}

void Location::postRotate(const GLfloat &x, const GLfloat &y, const GLfloat &z, GLfloat theta)
{
	theta += (GLfloat)(PIx2 * floor(theta / PIx2));

	mUpdate.P();
	mTransform.postRotate(x, y, z, theta);
	mUpdate.V();

	bTransform = true;
}


void Location::preRotate(GLfloat phi, GLfloat theta, GLfloat psi)
{
	phi += (GLfloat)(PIx2 * floor(phi / PIx2));
	theta += (GLfloat)(PIx2 * floor(theta / PIx2));
	psi += (GLfloat)(PIx2 * floor(psi / PIx2));

	TransformationMatrix	transform(TransformationMatrix::getRotationX(phi));
	transform.multiplyRight(TransformationMatrix::getRotationY(theta));
	transform.multiplyRight(TransformationMatrix::getRotationZ(psi));

	mUpdate.P();
	mTransform = mTransform * transform;
	mUpdate.V();

	bTransform = true;
}

void Location::postRotate(GLfloat phi, GLfloat theta, GLfloat psi)
{
	phi += (GLfloat)(PIx2 * floor(phi / PIx2));
	theta += (GLfloat)(PIx2 * floor(theta / PIx2));
	psi += (GLfloat)(PIx2 * floor(psi / PIx2));

	TransformationMatrix	transform(TransformationMatrix::getRotationX(phi));
	transform.multiplyRight(TransformationMatrix::getRotationY(theta));
	transform.multiplyRight(TransformationMatrix::getRotationZ(psi));

	mUpdate.P();
	mTransform = transform * mTransform;
	mUpdate.V();

	bTransform = true;
}

void Location::preRotateX(GLfloat phi)
{
	phi += (GLfloat)(PIx2 * floor(phi / PIx2));

	mUpdate.P();
	mTransform.preRotateX(phi);
	mUpdate.V();

	bTransform = true;
}

void Location::postRotateX(GLfloat phi)
{
	phi += (GLfloat)(PIx2 * floor(phi / PIx2));

	mUpdate.P();
	mTransform.postRotateX(phi);
	mUpdate.V();

	bTransform = true;
}

void Location::preRotateY(GLfloat theta)
{
	theta += (GLfloat)(PIx2 * floor(theta / PIx2));

	mUpdate.P();
	mTransform.preRotateY(theta);
	mUpdate.V();

	bTransform = true;
}

void Location::postRotateY(GLfloat theta)
{
	theta += (GLfloat)(PIx2 * floor(theta / PIx2));

	mUpdate.P();
	mTransform.postRotateY(theta);
	mUpdate.V();

	bTransform = true;
}

void Location::preRotateZ(GLfloat psi)
{
	psi += (GLfloat)(PIx2 * floor(psi / PIx2));

	mUpdate.P();
	mTransform.preRotateZ(psi);
	mUpdate.V();

	bTransform = true;
}

void Location::postRotateZ(GLfloat psi)
{
	psi += (GLfloat)(PIx2 * floor(psi / PIx2));

	mUpdate.P();
	mTransform.postRotateZ(psi);
	mUpdate.V();

	bTransform = true;
}


Location::Vector Location::getPosition()
{
	Vector	result;

	mUpdate.P();
	result.set(	mTransform.getElement(0, 3),
				mTransform.getElement(1, 3),
				mTransform.getElement(2, 3)	);
	mUpdate.V();

	return result;
}
Location::VectorD Location::getPositionD()
{
	VectorD	result;

	mLock.P();
	mUpdate.P();
	result.set(	mTransform.getElement(0, 3),
				mTransform.getElement(1, 3),
				mTransform.getElement(2, 3)	);
	mUpdate.V();
	mLock.V();

	return result;
}


Location::Vector Location::getForward()
{
	_transform();

	mLock.P();
	Vector	v(vForward);
	mLock.V();

	return v;
}

Location::VectorD Location::getForwardD()
{
	_transform();

	mLock.P();
	VectorD	v(vForward.getX(), vForward.getY(), vForward.getZ());
	mLock.V();

	return v;
}

Location::Vector Location::getUp()
{
	_transform();

	mLock.P();
	Vector	v(vUp);
	mLock.V();

	return v;
}

Location::VectorD Location::getUpD()
{
	_transform();

	mLock.P();
	VectorD	v(vUp.getX(), vUp.getY(), vUp.getZ());
	mLock.V();

	return v;
}


void Location::glApply()
{
	mUpdate.P();
	glMultTransposeMatrixf(mTransform.getPointer());
	mUpdate.V();
}


GRAPHICS_END
