#include "Map.h"


GRAPHICS_BEGIN


bool	Map::Data::bFadeTextures = true;
GLfloat	Map::Data::fLightX = 0.0f,
		Map::Data::fLightY = 1.0f,
		Map::Data::fLightZ = 0.0f,
		Map::Data::fLightMin = 0.0f;


Map::Data::Data(	const uint_t &_nMapSizeX,
					const uint_t &_nMapSizeZ,
					const uint8_t &_nBlockSizeExponent,
					const GLfloat &_fScaleX,
					const GLfloat &_fScaleY,
					const GLfloat &_fScaleZ				) :
	pHeightMap(NULL),
	nMapSizeX(_nMapSizeX),
	nMapSizeZ(_nMapSizeZ),
	nHeightMapX(_nMapSizeX + 1),
	nBlockSizeExponent(_getBlockSizeExponent(_nBlockSizeExponent)),
	fScaleX(_fScaleX),
	fScaleY(_fScaleY),
	fScaleZ(_fScaleZ),
	nIndexId(0),
	pIndices(NULL),
	pCornerBottomLeft(NULL)
{
	pHeightMap = new uint16_t[(nMapSizeX + 1) * (nMapSizeZ + 1)];
}

Map::Data::~Data()
{
	if(pHeightMap)
		delete[] pHeightMap;
	if(pIndices)
		delete[] pIndices;
	if(pCornerBottomLeft)
		delete[] pCornerBottomLeft;
}


void Map::Data::generateStrips()
{
	const GLushort	nBlockSize = getBlockSize(),
					nLineWidth = nBlockSize + 1,
					lvc = getLoDVertexCount();
	GLushort		step, step2,
					v1, v2, wb, wr, wt, wl;
	uint_t			pos = 0;

	// calculate level-of-detail triangle strips start/length
	for(uint_t i = 0; i < LOD_COUNT; i++)
	{
		nStartIndex[i] = pos;
		nLineLength[i] = ((2 << (nBlockSizeExponent - i)) - 2);

		pos =	nStartIndex[i] +
				// inner vertices
				nLineLength[i] * ((1 << (nBlockSizeExponent - i)) - 2) +
				// borders, multiplied by the leves of detail
				((nLineLength[i] << 2) + 2) * (LOD_COUNT - i);
	}
	nStartIndex[LOD_COUNT] = pos;

	// allocate index memory
	pIndices = new GLushort[pos];
	pCornerBottomLeft = new GLushort[getLoDCornerCount() << 3];
	pCornerBottomRight = pCornerBottomLeft + (getLoDCornerCount() << 1);
	pCornerTopRight = pCornerBottomRight + (getLoDCornerCount() << 1);
	pCornerTopLeft = pCornerTopRight + (getLoDCornerCount() << 1);

	// generate triange strip indices
	pos = 0;
	for(uint_t i = 0; i < LOD_COUNT; i++)
	{
		// the "inner" strips
		step = (GLushort)1 << i;
		for(GLushort z = step; z < nBlockSize - step; z += step)
			for(GLushort x = step; x < nBlockSize; x += step)
			{
				pIndices[pos++] = (GLushort)(z * nLineWidth + x);
				pIndices[pos++] = (GLushort)((z + step) * nLineWidth + x);
			}

		// LoD interpolated vertices
		wb = nLineWidth * nLineWidth/* + getLoDVertexIndex(i)*/;
		for(uint_t j = 1; j < i; j++)
			wb += (1 << nBlockSizeExponent) - (1 << (nBlockSizeExponent - j));
		wr = wb + lvc;
		wt = wr + lvc;
		wl = wt + lvc;


		// - skip the corners => start at step, end at block size - step
		// - calculate interpolated indices to the first "normal" element
		// - after that repeatedly generate the "normal" element, follow by the
		//   interpolated ones until before the next "normal" element
		// - finally compute the last interpolated vertices

		// bottom
		for(uint_t j = i, cpos = getLoDCornerIndex(i) << 1; j < LOD_COUNT; j++, cpos += 2)
		{
			step2 = 1 << j;
			v2 = nLineWidth * nBlockSize + step;	// last line
			v1 = v2 - (nLineWidth << i);			// next to last line

			wb += step - 1;
			for(GLushort _x = step; _x < step2; _x += step)
			{
				pIndices[pos++] = v1;				v1 += step;
				pIndices[pos++] = wb;	wb += step;	v2 += step;
			}
			for(GLushort x = step2; x < nBlockSize - step2; x += step2)
			{
				pIndices[pos++] = v1;	v1 += step;
				pIndices[pos++] = v2;	v2 += step;
				wb += step - 1;
				for(GLushort _x = step; _x < step2; _x += step)
				{
					pIndices[pos++] = v1;				v1 += step;
					pIndices[pos++] = wb;	wb += step;	v2 += step;
				}
			}
			pIndices[pos++] = v1;	v1 += step;
			pIndices[pos++] = v2;	v2 += step;
			wb += step - 1;
			for(GLushort _x = step; _x < step2; _x += step)
			{
				pIndices[pos++] = v1;				v1 += step;
				pIndices[pos++] = wb;	wb += step;	v2 += step;
			}

			pCornerBottomLeft[cpos + 1] = pIndices[pos - (1 << (nBlockSizeExponent - i + 1)) + 3];
			pCornerBottomRight[cpos] = pIndices[pos - 1];
		}

		// right
		for(uint_t j = i, cpos = getLoDCornerIndex(i) << 1; j < LOD_COUNT; j++, cpos += 2)
		{
			step2 = 1 << j;
			v1 = (nLineWidth << i) + nBlockSize - step;		// second row, next to last column

			// need the first vertex twice because the first triangle would be clockwise otherwise!
			pIndices[pos++] = v1;

			wr += step - 1;
			for(GLushort _z = step; _z < step2; _z += step)
			{
				pIndices[pos++] = v1;	v1 += nLineWidth << i;
				pIndices[pos++] = wr;	wr += step;
			}
			for(GLushort z = step2; z < nBlockSize - step2; z += step2)
			{
				pIndices[pos++] = v1;
				pIndices[pos++] = v1 + step;	v1 += nLineWidth << i;
				wr += step - 1;
				for(GLushort _z = step; _z < step2; _z += step)
				{
					pIndices[pos++] = v1;	v1 += nLineWidth << i;
					pIndices[pos++] = wr;	wr += step;
				}
			}
			pIndices[pos++] = v1;
			pIndices[pos++] = v1 + step;	v1 += nLineWidth << i;
			wr += step - 1;
			for(GLushort _z = step; _z < step2; _z += step)
			{
				pIndices[pos++] = v1;	v1 += nLineWidth << i;
				pIndices[pos++] = wr;	wr += step;
			}

			pCornerTopRight[cpos + 1] = pIndices[pos - (1 << (nBlockSizeExponent - i + 1)) + 3];
			pCornerBottomRight[cpos + 1] = pIndices[pos - 1];
		}

		// top
		for(uint_t j = i, cpos = getLoDCornerIndex(i) << 1; j < LOD_COUNT; j++, cpos += 2)
		{
			step2 = 1 << j;
			v1 = step;						// first line
			v2 = (nLineWidth << i) + step;	// second line

			wt += step - 1;
			for(GLushort _x = step; _x < step2; _x += step)
			{
				pIndices[pos++] = wt;	wt += step;	v1 += step;
				pIndices[pos++] = v2;				v2 += step;
			}
			for(GLushort x = step2; x < nBlockSize - step2; x += step2)
			{
				pIndices[pos++] = v1;	v1 += step;
				pIndices[pos++] = v2;	v2 += step;
				wt += step - 1;
				for(GLushort _x = step; _x < step2; _x += step)
				{
					pIndices[pos++] = wt;	wt += step;	v1 += step;
					pIndices[pos++] = v2;				v2 += step;
				}
			}
			pIndices[pos++] = v1;	v1 += step;
			pIndices[pos++] = v2;	v2 += step;
			wt += step - 1;
			for(GLushort _x = step; _x < step2; _x += step)
			{
				pIndices[pos++] = wt;	wt += step;	v1 += step;
				pIndices[pos++] = v2;				v2 += step;
			}

			pCornerTopLeft[cpos + 1] = pIndices[pos - (1 << (nBlockSizeExponent - i + 1)) + 2];
			pCornerTopRight[cpos] = pIndices[pos - 2];
		}

		// left
		for(uint_t j = i, cpos = getLoDCornerIndex(i) << 1; j < LOD_COUNT; j++, cpos += 2)
		{
			step2 = 1 << j;
			v1 = nLineWidth << i;		// second row, first column

			// need the first vertex twice because the first triangle would be clockwise otherwise!
			pIndices[pos++] = v1;

			wl += step - 1;
			for(GLushort _z = step; _z < step2 ; _z += step)
			{
				pIndices[pos++] = wl;
				pIndices[pos++] = v1 + step;	wl += step;	v1 += nLineWidth << i;
			}
			for(GLushort z = step2; z < nBlockSize - step2; z += step2)
			{
				pIndices[pos++] = v1;
				pIndices[pos++] = v1 + step;	v1 += nLineWidth << i;
				wl += step - 1;
				for(GLushort _z = step; _z < step2 ; _z += step)
				{
					pIndices[pos++] = wl;
					pIndices[pos++] = v1 + step;	wl += step;	v1 += nLineWidth << i;
				}
			}
			pIndices[pos++] = v1;
			pIndices[pos++] = v1 + step;	v1 += nLineWidth << i;
			wl += step - 1;
			for(GLushort _z = step; _z < step2 ; _z += step)
			{
				pIndices[pos++] = wl;
				pIndices[pos++] = v1 + step;	wl += step;	v1 += nLineWidth << i;
			}

			pCornerTopLeft[cpos] = pIndices[pos - (1 << (nBlockSizeExponent - i + 1)) + 1];
			pCornerBottomLeft[cpos] = pIndices[pos - 2];
		}

		nInnerCornerIndex[0 + i] = nLineWidth * (nBlockSize - step) + step;
		nInnerCornerIndex[LOD_COUNT + i] = nLineWidth * (nBlockSize - step) + nBlockSize - step;
		nInnerCornerIndex[2 * LOD_COUNT + i] = (nLineWidth << i) + nBlockSize - step;
		nInnerCornerIndex[3 * LOD_COUNT + i] = (nLineWidth << i) + step;
	}

	// corners
	nCornerIndex[0] = nLineWidth * nBlockSize;
	nCornerIndex[1] = nCornerIndex[0] + nBlockSize;
	nCornerIndex[2] = nBlockSize;
	nCornerIndex[3] = 0;
}


// generate upper left face normal and lower right face normal for quad X
// (in the current line, adjusted by the value in _heights)
#define _GENERATE_FACE_NORMALS(X)\
	nx = fScaleYZ * (_heights[X] - _heights[(X) + 1]);\
	nz = fScaleXY * (_heights[X] - _heights[(X) + pMapData->nHeightMapX]);\
	mag = sqrt(nx * nx + ny * ny + nz * nz);\
	\
	pFaceNormals[n++] = nx / mag;	pFaceNormals[n++] = ny / mag;	pFaceNormals[n++] = nz / mag;\
	\
	nx = fScaleYZ * (_heights[(X) + pMapData->nHeightMapX] - _heights[(X) + 1 + pMapData->nHeightMapX]);\
	nz = fScaleXY * (_heights[(X) + 1] - _heights[(X) + 1 + pMapData->nHeightMapX]);\
	mag = sqrt(nx * nx + ny * ny + nz * nz);\
	\
	pFaceNormals[n++] = nx / mag;	pFaceNormals[n++] = ny / mag;	pFaceNormals[n++] = nz / mag

void Map::Block::_generateVertexNormals()
{
	// calculate face normals, vertices and vertex normals
	const uint_t	nBlockSize = pMapData->getBlockSize(),
					nLeft = nBlockX ? nBlockX - 1 : nBlockX,
					nTop = nBlockZ ? nBlockZ - 1 : nBlockZ,
					nRight = (nBlockX + nBlockSize < pMapData->nMapSizeX) ? nBlockX + nBlockSize : nBlockX + nBlockSize - 1,
					nBottom = (nBlockZ + nBlockSize < pMapData->nMapSizeZ) ? nBlockZ + nBlockSize : nBlockZ + nBlockSize - 1;
	const GLfloat	fScaleXY = pMapData->fScaleX * pMapData->fScaleY,
					fScaleYZ = pMapData->fScaleY * pMapData->fScaleZ;

	uint_t			n = 0;
	GLfloat			nx,
					ny = pMapData->fScaleZ * pMapData->fScaleX,
					nz,
					mag;
	const uint16_t	*_heights;


	// allocate face normal buffer
	GLfloat *pFaceNormals = new GLfloat[(nBlockSize + 2) * (nBlockSize + 2) * 6];

	// generate first row
	_heights = pMapData->pHeightMap + nTop * pMapData->nHeightMapX;
	_GENERATE_FACE_NORMALS(nLeft);
	for(uint_t x = nBlockX; x < nBlockX + nBlockSize; x++)
	{
		_GENERATE_FACE_NORMALS(x);
	}
	_GENERATE_FACE_NORMALS(nRight);

	// generate normal rows (we don't need to bother with nTop or nBottom here...)
	_heights = pMapData->pHeightMap + nBlockZ * pMapData->nHeightMapX;
	for(uint_t z = nBlockZ; z < nBlockZ + nBlockSize; z++, _heights += pMapData->nHeightMapX)
	{
		_GENERATE_FACE_NORMALS(nLeft);
		for(uint_t x = nBlockX; x < nBlockX + nBlockSize; x++)
		{
			_GENERATE_FACE_NORMALS(x);
		}
		_GENERATE_FACE_NORMALS(nRight);
	}

	// generate last row
	_heights = pMapData->pHeightMap + nBottom * pMapData->nHeightMapX;
	_GENERATE_FACE_NORMALS(nLeft);
	for(uint_t x = nBlockX; x < nBlockX + nBlockSize; x++)
	{
		_GENERATE_FACE_NORMALS(x);
	}
	_GENERATE_FACE_NORMALS(nRight);


	// now calculate the vertex normals from the face normals
	GLfloat	*_fn1 = pFaceNormals,
			*_fn2 = _fn1 + (nBlockSize + 2) * 6,
			*_normals = pVertices + nVertices * 3;

	n = 0;
	for(uint_t z = 0; z <= nBlockSize; z++, _fn1 += 6, _fn2 += 6)
		for(uint_t x = 0; x <= nBlockSize; x++, _fn1 += 6, _fn2 += 6)
		{
			nx = _fn1[3] + _fn1[6] + _fn1[9]  + _fn2[0] + _fn2[3] + _fn2[6];
			ny = _fn1[4] + _fn1[7] + _fn1[10] + _fn2[1] + _fn2[4] + _fn2[7];
			nz = _fn1[5] + _fn1[8] + _fn1[11] + _fn2[2] + _fn2[5] + _fn2[8];
			mag = sqrt(nx * nx + ny * ny + nz * nz);

			_normals[n++] = nx / mag;	_normals[n++] = ny / mag;	_normals[n++] = nz / mag;
		}

	delete[] pFaceNormals;
}


#define _GENERATE_ALPHA()\
	if(_normals[1] < 0.3f)\
		*_slopes = 1.0f;\
	else if(_normals[1] > 0.7f)\
		*_slopes = 0.0f;\
	else\
		*_slopes = 1.0f - (_normals[1] - 0.3f) / 0.6f

void Map::Block::_generateAlpha()
{
	const GLfloat	lx = pMapData->fLightX,
					ly = pMapData->fLightY,
					lz = pMapData->fLightZ;
	GLfloat			*_slopes,
					*_normals,
					*_normals2,
					*_light,
					nx,
					ny,
					nz;

	_slopes = pVertices + nVertices * 8 + 3;
	_normals = pVertices + nVertices * 3;
	_normals2 = _normals + (pMapData->getBlockSize() + 1) * 3;
	_light = pLightMap = new GLfloat[1 << (pMapData->nBlockSizeExponent << 1)];

	for(uint_t z = 0; z < pMapData->getBlockSize(); z++, _slopes += 4, _normals += 3, _normals2 += 3)
	{
		for(uint_t x = 0; x < pMapData->getBlockSize(); x++, _slopes += 4, _normals += 3, _normals2 += 3, _light++)
		{
			nx = (_normals[0] + _normals[3] + _normals2[0] + _normals2[3]) / 4.0f;
			ny = (_normals[1] + _normals[4] + _normals2[1] + _normals2[4]) / 4.0f;
			nz = (_normals[2] + _normals[5] + _normals2[2] + _normals2[5]) / 4.0f;
			if(((nx + lx) * (nx + lx) + (ny + ly) * (ny + ly) + (nz + lz) * (nz + lz)) > 2.0f)
				*_light = lx * nx + ly * ny + lz * nz;
			else
				*_light = 0.0f;
			*_light = 1.0f - fabs(*_light) * (1.0f - pMapData->fLightMin) - pMapData->fLightMin;

			_GENERATE_ALPHA();
		}
		_GENERATE_ALPHA();
	}

	for(uint_t x = 0; x < pMapData->getBlockSize(); x++, _slopes += 4, _normals += 3)
	{
		_GENERATE_ALPHA();
	}
	_GENERATE_ALPHA();
}

void Map::Block::_load()
{
	// find minimum and maximum heigt of the current block
	// this is used to "centre" the block and minimise the bounding box
	const uint_t	nBlockSize = pMapData->getBlockSize();
	const uint16_t	*_heights;
	uint16_t		min = pMapData->pHeightMap[nBlockZ * pMapData->nHeightMapX + nBlockX],
					max = min;

	// generate vertex coordinates and texture coordinates
	_heights = pMapData->pHeightMap + nBlockZ * pMapData->nHeightMapX;
	for(uint_t _z = nBlockZ; _z <= nBlockZ + nBlockSize; _z++, _heights += pMapData->nHeightMapX)
		for(uint_t _x = nBlockX; _x <= nBlockX + nBlockSize; _x++)
		{
			const uint16_t	&val = _heights[_x];

			if(val < min)
				min = val;
			else if(val > max)
				max = val;
		}

	const GLfloat	fMin = min * pMapData->fScaleY,
					fMax = max * pMapData->fScaleY,
					fX = ((nBlockSize + 1) / -2.0f - nBlockX + (nBlockX + ((GLfloat)nBlockSize - pMapData->nMapSizeX) / 2.0f)) * pMapData->fScaleX ,
					fZ = ((nBlockSize + 1) / -2.0f - nBlockZ + (nBlockZ + ((GLfloat)nBlockSize - pMapData->nMapSizeZ) / 2.0f)) * pMapData->fScaleZ;

	// set location
	reset();
	updateFinal();

	// set bounding box
	boundingBox(	nBlockSize * pMapData->fScaleX, fMax - fMin, nBlockSize * pMapData->fScaleZ,
					(nBlockX + ((GLfloat)nBlockSize - pMapData->nMapSizeX) / 2.0f) * pMapData->fScaleX,
					(fMin + fMax) / 2.0f,
					(nBlockZ + ((GLfloat)nBlockSize - pMapData->nMapSizeZ) / 2.0f) * pMapData->fScaleZ	);

	// allocate vertex memory
	//	vertex buffer object data:
	//		"normal" vertex data
	//		LoD border vertex data (* 4)
	uint_t			v = 0,
					t = 0,
					c = 0;

	nVertices =	(nBlockSize + 1) * (nBlockSize + 1) +
				(pMapData->getLoDVertexCount() << 2);
	pVertices = new GLfloat[nVertices * 12];

	GLfloat *pNormals = pVertices + nVertices * 3,
			*pTextureCoords = pVertices + nVertices * 6,
			*pColors = pVertices + nVertices * 8,
			texS,
			texT = 0.0f,
			texS2,
			texT2 = 0.0f,
			tex2step = 1.0f / nBlockSize;

	// generate vertex coordinates and texture coordinates
	_heights = pMapData->pHeightMap + nBlockZ * pMapData->nHeightMapX;
	for(uint_t z = nBlockZ; z <= nBlockZ + nBlockSize; z++, _heights += pMapData->nHeightMapX, texT += 0.125f, texT2 += tex2step)
	{
		texS = 0.0f;
		texS2 = 0.0f;
		for(uint_t x = nBlockX; x <= nBlockX + nBlockSize; x++, texS += 0.125f, texS2 += tex2step)
		{
			pVertices[v++] = fX + x * pMapData->fScaleX;
			pVertices[v++] = _heights[x] * pMapData->fScaleY;
			pVertices[v++] = fZ + z * pMapData->fScaleZ;

			pTextureCoords[t++] = texS;
			pTextureCoords[t++] = texT;
			
			pColors[c++] = texS2;
			pColors[c++] = texT2;
			pColors[c++] = 0.0;
			c++;
		}
	}

	// generate normal vectors from the coordinates calculated above
	_generateVertexNormals();

	// calculate alpha value for terrain blending using the normal vectors
	_generateAlpha();


	// calculate level-of-detail terrain roughness
	const uint16_t	*__heights;
	uint_t			step,
					nextRow;
	float			A, B, C, D,
					s, xs, zs,
					h;

	for(uint_t i = 1; i < LOD_COUNT; i++)
	{
		fRoughness[i] = 0.0f;

		step = 1 << i;
		s = 1.0f / step;
		nextRow = pMapData->nHeightMapX << i;

		// roughness
		for(uint_t _z = nBlockZ; _z < nBlockZ + nBlockSize; _z += step)
		{
			_heights = pMapData->pHeightMap + _z * pMapData->nHeightMapX;
			for(uint_t _x = nBlockX; _x < nBlockX + nBlockSize; _x += step)
			{
				D = _heights[_x];
				C = _heights[_x + step];
				B = _heights[nextRow + _x + step];
				A = _heights[nextRow + _x];

				for(uint_t z = 0; z < step; z++)
 				{
					__heights = _heights + z * pMapData->nHeightMapX + _x;
					for(uint_t x = 0; x < step; x++)
 					{
						xs = x * s;
						zs = z * s;
						h =	A * (1.0f - xs) * zs +
							B * xs * zs +
							C * xs * (1.0f - zs) +
							D * (1.0f - xs) * (1.0f - zs) -
							__heights[x];
						fRoughness[i] += h * h;
					}
				}
			}
		}

		fRoughness[i] *= pMapData->fScaleY / (pMapData->fScaleX * pMapData->fScaleZ * nBlockSize * nBlockSize);
	}


	// calculate LoD borders
	uint_t	v2, t2, c2,
			vline = 3 * (nBlockSize) + 1,
			tline = 2 * (nBlockSize) + 1,
			cline = 4 * (nBlockSize) + 1;

	// bottom
	_heights = pMapData->pHeightMap + (nBlockZ + nBlockSize) * pMapData->nHeightMapX;
	for(uint_t i = 1; i < LOD_COUNT; i++)
	{
		step = 1 << i;
		s = 1.0f / step;

		v2 = ((nBlockSize + 1) * 3) << pMapData->nBlockSizeExponent;
		t2 = ((nBlockSize + 1) * 2) << pMapData->nBlockSizeExponent;
		c2 = ((nBlockSize + 1) * 4) << pMapData->nBlockSizeExponent;

		for(uint_t _x = nBlockX; _x < nBlockX + nBlockSize; _x += step)
		{
			B = _heights[_x + step];
			A = _heights[_x];

			v2 += 3;
			t2 += 2;
			c2 += 4;

			for(float xs = s; xs < 1.0f; xs += s)
			{
				pVertices[v] = pVertices[v2];
				pNormals[v] = pNormals[v2];										v++;	v2++;
				pVertices[v] = (A * (1.0f - xs) + B * xs) * pMapData->fScaleY;
				pNormals[v] = pNormals[v2];										v++;	v2++;
				pVertices[v] = pVertices[v2];
				pNormals[v] = pNormals[v2];										v++;	v2++;
				pTextureCoords[t] = pTextureCoords[t2];							t++;	t2++;
				pTextureCoords[t] = pTextureCoords[t2];							t++;	t2++;
				pColors[c] = pColors[c2];										c++;	c2++;
				pColors[c] = pColors[c2];										c++;	c2++;
				pColors[c] = pColors[c2];										c++;	c2++;
				pColors[c] = pColors[c2];										c++;	c2++;
			}
		}
	}

	// right
	_heights = pMapData->pHeightMap + nBlockZ * pMapData->nHeightMapX + nBlockX + nBlockSize;
	for(uint_t i = 1; i < LOD_COUNT; i++)
	{
		step = 1 << i;
		s = 1.0f / step;
		nextRow = pMapData->nHeightMapX << i;

		v2 = 3 * nBlockSize;
		t2 = 2 * nBlockSize;
		c2 = 4 * nBlockSize;

		__heights = _heights;
		for(uint_t _z = nBlockZ; _z < nBlockZ + nBlockSize; _z += step, __heights += nextRow)
		{
			C = __heights[0];
			B = __heights[nextRow];

			v2 += vline + 2;
			t2 += tline + 1;
			c2 += cline + 3;

			for(float zs = s; zs < 1.0f; zs += s)
			{
				pVertices[v] = pVertices[v2];
				pNormals[v] = pNormals[v2];										v++;	v2++;
				pVertices[v] = (B * zs + C * (1.0f - zs)) * pMapData->fScaleY;
				pNormals[v] = pNormals[v2];										v++;	v2++;
				pVertices[v] = pVertices[v2];
				pNormals[v] = pNormals[v2];										v++;	v2 += vline;
				pTextureCoords[t] = pTextureCoords[t2];							t++;	t2++;
				pTextureCoords[t] = pTextureCoords[t2];							t++;	t2 += tline;
				pColors[c] = pColors[c2];										c++;	c2++;
				pColors[c] = pColors[c2];										c++;	c2++;
				pColors[c] = pColors[c2];										c++;	c2++;
				pColors[c] = pColors[c2];										c++;	c2 += cline;
			}
		}
	}

	// top
	_heights = pMapData->pHeightMap + nBlockZ * pMapData->nHeightMapX;
	for(uint_t i = 1; i < LOD_COUNT; i++)
	{
		step = 1 << i;
		s = 1.0f / step;

		v2 = 0;
		t2 = 0;
		c2 = 0;

		for(uint_t _x = nBlockX; _x < nBlockX + nBlockSize; _x += step)
		{
			D = _heights[_x];
			C = _heights[_x + step];

			v2 += 3;
			t2 += 2;
			c2 += 4;

			for(float xs = s; xs < 1.0f; xs += s)
			{
				pVertices[v] = pVertices[v2];
				pNormals[v] = pNormals[v2];										v++;	v2++;
				pVertices[v] = (C * xs + D * (1.0f - xs)) * pMapData->fScaleY;
				pNormals[v] = pNormals[v2];										v++;	v2++;
				pVertices[v] = pVertices[v2];
				pNormals[v] = pNormals[v2];										v++;	v2++;
				pTextureCoords[t] = pTextureCoords[t2];							t++;	t2++;
				pTextureCoords[t] = pTextureCoords[t2];							t++;	t2++;
				pColors[c] = pColors[c2];										c++;	c2++;
				pColors[c] = pColors[c2];										c++;	c2++;
				pColors[c] = pColors[c2];										c++;	c2++;
				pColors[c] = pColors[c2];										c++;	c2++;
			}
		}
	}

	// left
	_heights = pMapData->pHeightMap + nBlockZ * pMapData->nHeightMapX + nBlockX;
	for(uint_t i = 1; i < LOD_COUNT; i++)
	{
		step = 1 << i;
		s = 1.0f / step;
		nextRow = pMapData->nHeightMapX << i;

		v2 = 0;
		t2 = 0;
		c2 = 0;

		__heights = _heights;
		for(uint_t _z = nBlockZ; _z < nBlockZ + nBlockSize; _z += step, __heights += nextRow)
		{
			D = __heights[0];
			A = __heights[nextRow];

			v2 += vline + 2;
			t2 += tline + 1;
			c2 += cline + 3;

			for(float zs = s; zs < 1.0f; zs += s)
			{
				pVertices[v] = pVertices[v2];
				pNormals[v] = pNormals[v2];										v++;	v2++;
				pVertices[v] = (A * zs + D * (1.0f - zs)) * pMapData->fScaleY;
				pNormals[v] = pNormals[v2];										v++;	v2++;
				pVertices[v] = pVertices[v2];
				pNormals[v] = pNormals[v2];										v++;	v2 += vline;
				pTextureCoords[t] = pTextureCoords[t2];							t++;	t2++;
				pTextureCoords[t] = pTextureCoords[t2];							t++;	t2 += tline;
				pColors[c] = pColors[c2];										c++;	c2++;
				pColors[c] = pColors[c2];										c++;	c2++;
				pColors[c] = pColors[c2];										c++;	c2++;
				pColors[c] = pColors[c2];										c++;	c2 += cline;
			}
		}
	}


	// load the textures
	if(!pMapData->texture0.isEmpty())
		pMapData->texture0->load();
	if(!pMapData->texture1.isEmpty())
		pMapData->texture1->load();
}

void Map::Block::_remove()
{
	if(pVertices != NULL)
	{
		delete[] pVertices;
		pVertices = NULL;
	}
	if(pLightMap != NULL)
	{
		delete[] pLightMap;
		pLightMap = NULL;
	}

	if(!pMapData->texture0.isEmpty())
		pMapData->texture0->remove();
	if(!pMapData->texture1.isEmpty())
		pMapData->texture1->remove();
}


void Map::Block::_glLoad()
{
	// generate an OpenGL Buffer Object
	glGenBuffersARB(1, &nVboId);
	if(!nVboId)
		return;

	// transfer data to buffer
	glBindBufferARB(GL_ARRAY_BUFFER_ARB, nVboId);
	glBufferDataARB(	GL_ARRAY_BUFFER_ARB,
						nVertices * 12 * sizeof(GLfloat),
						pVertices,
						GL_STATIC_DRAW_ARB																			);

	// generate lightmap texture object
	glGenTextures(1, &nLightMapId);
	glBindTexture(GL_TEXTURE_2D, nLightMapId);

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	gluBuild2DMipmaps(	GL_TEXTURE_2D,
						1,
						pMapData->getBlockSize(),
						pMapData->getBlockSize(),
						GL_LUMINANCE,
						GL_FLOAT,
						pLightMap					);

	if(!pMapData->texture0.isEmpty())
		pMapData->texture0->glLoad();
	if(!pMapData->texture1.isEmpty())
		pMapData->texture1->glLoad();

	if(!pMapData->nIndexId)
	{
		glGenBuffersARB(1, &(pMapData->nIndexId));
		if(!pMapData->nIndexId)
			return;

		glBindBufferARB(GL_ELEMENT_ARRAY_BUFFER_ARB, pMapData->nIndexId);
		glBufferDataARB(	GL_ELEMENT_ARRAY_BUFFER_ARB,
							(pMapData->nStartIndex[LOD_COUNT]) * sizeof(GLushort),
							pMapData->pIndices,
							GL_STATIC_DRAW_ARB										);

	}
}

void Map::Block::_glRemove()
{
	if(nVboId)
	{	glDeleteBuffersARB(1, &nVboId);
		nVboId = 0;							}

	if(nLightMapId != 0)
		glDeleteTextures(1, &nLightMapId);

	if(!pMapData->texture0.isEmpty())
		pMapData->texture0->glRemove();
	if(!pMapData->texture1.isEmpty())
		pMapData->texture1->glRemove();

	if(pMapData->nIndexId)
	{	glDeleteBuffersARB(1, &(pMapData->nIndexId));
		pMapData->nIndexId = 0;							}
}


void Map::Block::_glRender()
{
	glBindBufferARB(GL_ARRAY_BUFFER_ARB, nVboId);

	glEnableClientState(GL_VERTEX_ARRAY);
	glVertexPointer(3, GL_FLOAT, 0, NULL);

	if(pMapData->bFadeTextures)
	{
		glDisable(GL_LIGHTING);

		glEnableClientState(GL_COLOR_ARRAY);
		glColorPointer(4, GL_FLOAT, 0, ((GLfloat *)0 + nVertices * 8));

		// Lightmap-Textur
		glActiveTexture(GL_TEXTURE2);
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_BLEND);
		glEnable(GL_TEXTURE_2D);
		glClientActiveTexture(GL_TEXTURE2);
		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
		glTexCoordPointer(2, GL_FLOAT, 4 * sizeof(GLfloat), ((GLfloat *)0 + nVertices * 8));
		glBindTexture(GL_TEXTURE_2D, nLightMapId);

		// Texturen fr Multitexturing der Map aktivieren
		glActiveTexture(GL_TEXTURE1);
		glClientActiveTexture(GL_TEXTURE1);
		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
		glTexCoordPointer(2, GL_FLOAT, 0, ((GLfloat *)0 + nVertices * 6));
		glEnable(GL_TEXTURE_2D);
		pMapData->texture1->glBind();
		
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE);
		glTexEnvf(GL_TEXTURE_ENV, GL_COMBINE_RGB, GL_INTERPOLATE);

		glTexEnvf(GL_TEXTURE_ENV, GL_SOURCE0_RGB, GL_TEXTURE);
		glTexEnvf(GL_TEXTURE_ENV, GL_OPERAND0_RGB, GL_SRC_COLOR);

		glTexEnvf(GL_TEXTURE_ENV, GL_SOURCE1_RGB, GL_PREVIOUS);
		glTexEnvf(GL_TEXTURE_ENV, GL_OPERAND1_RGB, GL_SRC_COLOR);

		glTexEnvf(GL_TEXTURE_ENV, GL_SOURCE2_RGB, GL_PRIMARY_COLOR);
		glTexEnvf(GL_TEXTURE_ENV, GL_OPERAND2_RGB, GL_SRC_ALPHA);
	}
	else
	{
		glEnableClientState(GL_NORMAL_ARRAY);
		glNormalPointer(GL_FLOAT, 0, ((GLfloat *)0 + nVertices * 3));
	}

	// Texture 0
	glActiveTexture(GL_TEXTURE0);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
	glEnable(GL_TEXTURE_2D);
	glClientActiveTexture(GL_TEXTURE0);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	glTexCoordPointer(2, GL_FLOAT, 0, ((GLfloat *)0 + nVertices * 6));
	pMapData->texture0->glBind();


	const GLushort	*pIndices = (GLushort *)0 + pMapData->nStartIndex[nLoD],
					*pBottomLoD = pIndices + pMapData->nLineLength[nLoD] * ((1 << (pMapData->nBlockSizeExponent - nLoD)) - 2),
					*pRightLoD = pBottomLoD + pMapData->nLineLength[nLoD] * (LOD_COUNT - nLoD),
					*pTopLoD = pRightLoD + (pMapData->nLineLength[nLoD] + 1) * (LOD_COUNT - nLoD),
					*pLeftLoD = pTopLoD + pMapData->nLineLength[nLoD] * (LOD_COUNT - nLoD),
					nLoDCornerIndex = pMapData->getLoDCornerIndex(nLoD) << 1;
	const uint8_t	nBottomLoD = pBottom ? (pBottom->nLoD > nLoD ? pBottom->nLoD - nLoD : 0) : 0,
					nRightLoD = pRight ? (pRight->nLoD > nLoD ? pRight->nLoD - nLoD : 0) : 0,
					nTopLoD = pTop ? (pTop->nLoD > nLoD ? pTop->nLoD - nLoD : 0) : 0,
					nLeftLoD = pLeft ? (pLeft->nLoD > nLoD ? pLeft->nLoD - nLoD : 0) : 0;

	// select the index buffer
	glBindBufferARB(GL_ELEMENT_ARRAY_BUFFER_ARB, pMapData->nIndexId);

	// draw the "inner" vertices
	for(uint_t i = 0; i < ((uint_t)1 << (pMapData->nBlockSizeExponent - nLoD)) - 2; i++)
		glDrawElements(GL_TRIANGLE_STRIP, pMapData->nLineLength[nLoD], GL_UNSIGNED_SHORT, pIndices + pMapData->nLineLength[nLoD] * i);

	// draw the borders
	glDrawElements(GL_TRIANGLE_STRIP, pMapData->nLineLength[nLoD], GL_UNSIGNED_SHORT, pBottomLoD + pMapData->nLineLength[nLoD] * nBottomLoD);
	glDrawElements(GL_TRIANGLE_STRIP, pMapData->nLineLength[nLoD] + 1, GL_UNSIGNED_SHORT, pRightLoD + (pMapData->nLineLength[nLoD] + 1) * nRightLoD);
	glDrawElements(GL_TRIANGLE_STRIP, pMapData->nLineLength[nLoD], GL_UNSIGNED_SHORT, pTopLoD + pMapData->nLineLength[nLoD] * nTopLoD);
	glDrawElements(GL_TRIANGLE_STRIP, pMapData->nLineLength[nLoD] + 1, GL_UNSIGNED_SHORT, pLeftLoD + (pMapData->nLineLength[nLoD] + 1) * nLeftLoD);

	// bottom left
	glBegin(GL_TRIANGLE_STRIP);
		glArrayElement(pMapData->pCornerBottomLeft[nLoDCornerIndex + (nLeftLoD << 1)]);
		glArrayElement(pMapData->nCornerIndex[0]);
		glArrayElement(pMapData->nInnerCornerIndex[nLoD]);
		glArrayElement(pMapData->pCornerBottomLeft[nLoDCornerIndex + (nBottomLoD << 1) + 1]);
	glEnd();

	// bottom right
	glBegin(GL_TRIANGLE_STRIP);
		glArrayElement(pMapData->nInnerCornerIndex[LOD_COUNT + nLoD]);
		glArrayElement(pMapData->pCornerBottomRight[nLoDCornerIndex + (nBottomLoD << 1)]);
		glArrayElement(pMapData->pCornerBottomRight[nLoDCornerIndex + (nRightLoD << 1) + 1]);
		glArrayElement(pMapData->nCornerIndex[1]);
	glEnd();

	// top right
	glBegin(GL_TRIANGLE_STRIP);
		glArrayElement(pMapData->pCornerTopRight[nLoDCornerIndex + (nTopLoD << 1)]);
		glArrayElement(pMapData->nInnerCornerIndex[2 * LOD_COUNT + nLoD]);
		glArrayElement(pMapData->nCornerIndex[2]);
		glArrayElement(pMapData->pCornerTopRight[nLoDCornerIndex + (nRightLoD << 1) + 1]);
	glEnd();

	// top left
	glBegin(GL_TRIANGLE_STRIP);
		glArrayElement(pMapData->nCornerIndex[3]);
		glArrayElement(pMapData->pCornerTopLeft[nLoDCornerIndex + (nLeftLoD << 1)]);
		glArrayElement(pMapData->pCornerTopLeft[nLoDCornerIndex + (nTopLoD << 1) + 1]);
		glArrayElement(pMapData->nInnerCornerIndex[3 * LOD_COUNT + nLoD]);
	glEnd();


	if(pMapData->bFadeTextures)
	{
		glClientActiveTexture(GL_TEXTURE2);
		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
		glActiveTexture(GL_TEXTURE2);
		glDisable(GL_TEXTURE_2D);

		glClientActiveTexture(GL_TEXTURE1);
		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
		glActiveTexture(GL_TEXTURE1);
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
		glDisable(GL_TEXTURE_2D);

		glClientActiveTexture(GL_TEXTURE0);
		glActiveTexture(GL_TEXTURE0);

		glDisableClientState(GL_COLOR_ARRAY);
		glEnable(GL_LIGHTING);
	}
	else
		glDisableClientState(GL_NORMAL_ARRAY);

	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);

	glDisableClientState(GL_VERTEX_ARRAY);
}


Map::Block::Block(Map *parent, const uint_t &blockx, const uint_t &blockz) :
	Object3(true),
	pMapData(&(parent->data)),
	nBlockX(blockx),
	nBlockZ(blockz),
	pBottom(NULL),
	pRight(NULL),
	pTop(NULL),
	pLeft(NULL),
	nLoD(0)
{	type(TERRAIN_CATHEGORY, TERRAIN_TYPE, parent);	}

Map::Block::~Block()
{	_remove();	}


void Map::Block::cameraPosition(const Vector &camera)
{
	const float	px = pBoundary[3],
				py = pBoundary[4],
				pz = pBoundary[5],
				cx = camera.getX(),
				cy = camera.getY(),
				cz = camera.getZ();
	float		distance;

	if(	(cx >= pBoundary[3] - pBoundary[0]) && (cx <= pBoundary[3] - pBoundary[0]) &&
		(cz >= pBoundary[5] - pBoundary[2]) && (cz <= pBoundary[5] - pBoundary[2])		)
	{
		distance = cy - ((Map *)pTypeData)->heightAt(cx, cz);
	}
	else
	{
		const float	x = pBoundary[3] - cx,
					y = pBoundary[4] - cy,
					z = pBoundary[5] - cz;

		distance = sqrt(x * x + y * y + z * z);
	}

	if(distance < 1.0f)
		distance = 1.0f;

	for(nLoD = 1; nLoD < LOD_COUNT; nLoD++)
		if(fRoughness[nLoD] / distance > 0.0075f)
			break;
	nLoD--;
}


void Map::Block::glReload()
{
	if(!pMapData->texture0.isEmpty())
		pMapData->texture0->glReload();
	if(!pMapData->texture1.isEmpty())
		pMapData->texture1->glReload();
}


void Map::Block::glBind()
{		}


Map::Map(	const string &_strFile,
			const uint_t &mapsizex,
			const uint_t &mapsizez,
			const uint8_t nBlockSizeExponent,
			const float &scalex,
			const float &scaley,
			const float &scalez					) :
	strFile(_strFile),
	data(mapsizex, mapsizez, nBlockSizeExponent, scalex, scaley, scalez)
{
	const uint_t	nBlockSize = data.getBlockSize();
	uint_t			i;
	Block			**previous_blocks = new Block *[data.nMapSizeX >> nBlockSizeExponent],
					*previous,
					*block;

	// reserve space for the block to avoid resizing
	blocks.reserve((data.nMapSizeX >> nBlockSizeExponent) * (data.nMapSizeZ >> nBlockSizeExponent));

	// initialise adjacent blocks for the top row to NULL
	for(i = 0; i < data.nMapSizeX >> nBlockSizeExponent; i++)
		previous_blocks[i] = NULL;

	for(int_t z = 0; z + nBlockSize <= data.nMapSizeZ; z += nBlockSize)
	{
		i = 0;
		previous = NULL;

		for(int_t x = 0; x + nBlockSize <= data.nMapSizeX; x += nBlockSize)
		{
			block = new Block(this, x, z);

			// connect the blocks (for the LoD borders)
			block->left(previous);
			if(previous)
				previous->right(block);
			previous = block;

			block->top(previous_blocks[i]);
			if(previous_blocks[i])
				previous_blocks[i]->bottom(block);
			previous_blocks[i++] = block;

			blocks.push_back(block);
		}
	}

	delete[] previous_blocks;
}

Map::~Map()
{		
	for(vector<_Object3 *>::iterator it = blocks.begin(); it != blocks.end(); it++)
		delete *it;
}


///////////////////////////////// LOAD RAW FILE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	Hier wird eine bestimmte Lnge eines .Raw-Bildes gelesen und die 
/////	Farbwerte als Hhenwerte in der Map gespeichert. Zustzlich wird 
/////	die Maximal hhe jedes einzelnen Map-Blocks errechnet (fr die 
/////	spter eingesetzte Level-Of-Detail-Funktion).
/////
///////////////////////////////// LOAD RAW FILE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
void Map::_load()
{
	FILE	*file = NULL;
	int		readSize = (data.nMapSizeX + 1) * (data.nMapSizeZ + 1);

	// ffen der Datei in Read/Binary-Mode.
	file = fopen(strFile.c_str(), "rb");
	if(file == NULL)
	{
		fclose(file);
		// Anzeigen einer Fehlermeldung und Beendigung der Funktion
		throw Exception("Hhen-Map konnte nicht gefunden werden!");
	}

	// Hier laden wir die .Raw-Datei in ein Buffer-Array.
	// Wir lesen hier nur '1' (=Byte), und readSize ist Maphhe * Mapbreite.
	fread(data.pHeightMap, sizeof(unsigned short), readSize, file);
	// berprft, ob ein Fehler aufgetaucht ist.
	if(ferror(file))
	{
		fclose(file);
		throw Exception("Es konnten keine Daten aus der Hhen-Map gelesen werden!");
	}

	// Schlieen der Datei.
	fclose(file);


	// triangle strips generieren
	data.generateStrips();

	// vertex-daten in den blcken erstellen
	for(vector<_Object3 *>::iterator it = blocks.begin(); it != blocks.end(); it++)
		(*it)->load();
}

void Map::_remove()
{
	for(vector<_Object3 *>::iterator it = blocks.begin(); it != blocks.end(); it++)
		(*it)->remove();

	if(data.pIndices != NULL)
	{
		delete[] data.pIndices;
		data.pIndices = NULL;
	}
	if(data.pCornerBottomLeft != NULL)
	{
		delete[] data.pCornerBottomLeft;
		data.pCornerBottomLeft = NULL;
	}
}


void Map::_glLoad()
{
	for(vector<_Object3 *>::iterator it = blocks.begin(); it != blocks.end(); it++)
		(*it)->glLoad();
}

void Map::_glRemove()
{
	for(vector<_Object3 *>::iterator it = blocks.begin(); it != blocks.end(); it++)
		(*it)->glRemove();
}


/****************************************************************/
// _heightAt:
//	Returns the (interpolated) height value for the coordinates
//	x and z. The interpolation is linear from the texture coordi-
//	nates (floor(x),floor(z)) to (ceil(x),ceil(z)), wheighted by
//	fpart(x) and fpart(z).
/****************************************************************/
float Map::_heightAt(float x, float z) const
{
	// get integer coordinates
	const uint_t	_x = (uint_t)x,
					_z = (uint_t)z;

	// the coordinates within the current quad
	x -= _x;
	z -= _z;

	const uint16_t	*_heights = data.pHeightMap + _z * data.nHeightMapX;
	const float		D = _heights[_x],
					C = _heights[_x + 1],
					B = _heights[data.nHeightMapX + _x + 1],
					A = _heights[data.nHeightMapX + _x];

	return	A * (1.0f - x) * z +
			B * x * z +
			C * x * (1.0f - z) +
			D * (1.0f - x) * (1.0f - z);
}


/****************************************************************/
// slopeAt:
//	computes the maximum slope at (x, z) using the differences
//	dx and dz.
/****************************************************************/
float Map::_slopeAt(float x, float z, const float &dx, const float &dz) const
{
	const GLfloat	x1 = x < dx ? 0 : x - dx,
					x2 = x + dx > data.nMapSizeX ? data.nMapSizeX : x + dx,
					z1 = z < dz ? 0 : z - dz,
					z2 = z + dz > data.nMapSizeZ ? data.nMapSizeZ : z + dz,
					sx = fabs(	_heightAt(x1, z) - _heightAt(x2, z)	) / (x2 - x1),
					sz = fabs(	_heightAt(x, z1) - _heightAt(x, z2)	) / (z2 - z1);

	return sx > sz ? sx : sz;
}


GRAPHICS_END
