#include <stdlib.h>
#include <assert.h>

#include <graphics/Material.h> 


GRAPHICS_BEGIN


Material	Material::standard;


void Material::_load()
{
	if(!rTexture.isEmpty())
		rTexture->load();
}

void Material::_remove()
{
	if(!rTexture.isEmpty())
		rTexture->remove();
}

void Material::_glLoad()
{
	if(!rTexture.isEmpty())
		rTexture->glLoad();
}

void Material::_glRemove()
{
	if(!rTexture.isEmpty())
		rTexture->glRemove();
}


Material::Material()
{
	Ambient[0] = 0.2f;
	Ambient[1] = 0.2f;
	Ambient[2] = 0.2f;
	Ambient[3] = 1.0f;

	Diffuse[0] = 0.8f;
	Diffuse[1] = 0.8f;
	Diffuse[2] = 0.8f;
	Diffuse[3] = 1.0f;

	Specular[0] = 0.0f;
	Specular[1] = 0.0f;
	Specular[2] = 0.0f;
	Specular[3] = 1.0f;

	Emissive[0] = 0.0f;
	Emissive[1] = 0.0f;
	Emissive[2] = 0.0f;
	Emissive[3] = 1.0f;

	Shininess = 0.0f;
}

Material::Material(float* _Ambient, float* _Diffuse, float* _Specular, float* _Emissive, float _Shininess)
{
	for(int i = 0; i <= 3; i++)
	{
		Ambient[i] = _Ambient[i];
		Diffuse[i] = _Diffuse[i];
		Specular[i] = _Specular[i];
		Emissive[i] = _Emissive[i];
	}

	Shininess = _Shininess;
}

Material::~Material()	
{	/* nothing to do */	}


Material* Material::loadFromMs3dAsciiSegment( FILE *file )
{
    char szLine[256];
	//bool bError;

	Material *material = new Material();

    // name
    if (!fgets (szLine, 256, file))
		return NULL;
    if (sscanf (szLine, "\"%[^\"]\"", material->Name) != 1)
		return NULL;

	// ambient
    if (!fgets (szLine, 256, file))
		return NULL;

	if (sscanf (szLine, "%f %f %f %f", &(material->Ambient[0]), &(material->Ambient[1]), &(material->Ambient[2]), &(material->Ambient[3])) != 4)
		return NULL;

    // diffuse
    if (!fgets (szLine, 256, file))
		return NULL;
    if (sscanf (szLine, "%f %f %f %f", &(material->Diffuse[0]), &(material->Diffuse[1]), &(material->Diffuse[2]), &(material->Diffuse[3])) != 4)
		return NULL;

    // specular
    if (!fgets (szLine, 256, file))
		return NULL;
    if (sscanf (szLine, "%f %f %f %f", &(material->Specular[0]), &(material->Specular[1]), &(material->Specular[2]), &(material->Specular[3])) != 4)
		return NULL;

    // emissive
    if (!fgets (szLine, 256, file))
		return NULL;
    if (sscanf (szLine, "%f %f %f %f", &(material->Emissive[0]), &(material->Emissive[1]), &(material->Emissive[2]), &(material->Emissive[3])) != 4)
		return NULL;

    // shininess
    if (!fgets (szLine, 256, file))
		return NULL;
    if (sscanf (szLine, "%f", &(material->Shininess)) != 1)
		return NULL;

    // transparency
    if (!fgets (szLine, 256, file))
		return NULL;
    if (sscanf (szLine, "%f", &(material->Transparency)) != 1)
		return NULL;

    // diffuse rTexture
    if (!fgets (szLine, 256, file))
		return NULL;
	strcpy(material->DiffuseTexture, "");
    sscanf (szLine, "\"%[^\"]\"", material->DiffuseTexture);

    // alpha rTexture
    if (!fgets (szLine, 256, file))
		return NULL;
	strcpy(material->AlphaTexture, "");
    sscanf (szLine, "\"%[^\"]\"", material->AlphaTexture);

	if( strlen(material->DiffuseTexture) > 0 )
		material->rTexture = Texture::create(material->DiffuseTexture);

	return material;
}


void Material::glReload()
{
	GlItem::glReload();

	if(!rTexture.isEmpty())
		rTexture->glReload();
}

void Material::glBind()
{
	if(!rTexture.isEmpty())
		rTexture->glBind();

	glMaterialfv(GL_FRONT, GL_AMBIENT, Ambient);
	glMaterialfv(GL_FRONT, GL_DIFFUSE, Diffuse);
	glMaterialfv(GL_FRONT, GL_SPECULAR, Specular);
	glMaterialfv(GL_FRONT, GL_EMISSION, Emissive);
	glMaterialf(GL_FRONT, GL_SHININESS, Shininess);
}


GRAPHICS_END
