#ifndef __GRAPHICS__OBJECT_H__INCLUDED__
#define __GRAPHICS__OBJECT_H__INCLUDED__


#include <framework/Reference.h>

#include "GlItem.h"
#include "TransformationMatrix4.h"


GRAPHICS_BEGIN


class Object :	public GlItem
{
public:
	typedef	TransformationMatrix4<GLfloat>	TransformationMatrix;


protected:
	// manage removing from scene
	bool		bDeleteOnRemove;

	// modeling transformations
	TransformationMatrix	*ptmParent,
							tmLocal,
							tmFinal,
							tmGL;


	// compute final transformation matrix
	inline void		updateFinal()
	{	tmFinal = ptmParent ? (*ptmParent) * tmLocal : tmLocal;	}


	// render the object
	virtual void	_glRender() = 0;


public:
	inline Object() :	ptmParent(NULL),
						bDeleteOnRemove(false)
	{	reset();	}


	// delete when removing from scene?
	inline void	deleteOnRemove(bool bDelete)
	{	bDeleteOnRemove = bDelete;	}

	inline bool	getDeleteOnRemove() const
	{	return bDeleteOnRemove;	}


	// transform a vector with the current modelling transformation
	inline Vector	getTransformed(const Vector &v) const
	{	return tmFinal * v;	}


	// transformations
	inline void		reset()
	{	tmLocal.unity();	}


	// advance simulation
	virtual bool	advance(float time);

	// OpenGL
	// update GL transformation matrix
	virtual void	glUpdate();
	// bind object data, position and render
	void			glRender();
};


GRAPHICS_END
#endif
