#include "Particle.h"
#include <math.h>
#include <stdio.h>


GRAPHICS_BEGIN


void Particle::_load()
{
	if(!m_tex_particletexture.isEmpty())
		m_tex_particletexture->load();
}

void Particle::_remove()
{
	if(!m_tex_particletexture.isEmpty())
		m_tex_particletexture->remove();
}


void Particle::_glLoad()
{
	if(!m_tex_particletexture.isEmpty())
		m_tex_particletexture->glLoad();
}

void Particle::_glRemove()
{
	if(!m_tex_particletexture.isEmpty())
		m_tex_particletexture->glLoad();
}


void Particle::_glRender()
{
	// Soll das Partikel berhaupt gezeichnet werden?
	float x[4], y[4], z[4];
	x[0] = -m_fl_size;
	y[0] = +m_fl_size;
	z[0] = 0;

	x[1] = -m_fl_size;
	y[1] = -m_fl_size;
	z[1] = 0;

	x[2] = +m_fl_size;
	y[2] = -m_fl_size;
	z[2] = 0;

	x[3] = +m_fl_size;
	y[3] = +m_fl_size;
	z[3] = 0;
	
	glPushAttrib(GL_ENABLE_BIT);

	glEnable(GL_BLEND);
	glDisable(GL_DEPTH_TEST);
	glDisable(GL_LIGHTING);

	// Setzen der Textur fr die Partikel-Flche.
	m_tex_particletexture->glBindLoad();		

	// Partikelfarbe setzen.
	glColor4f(m_fl_r, m_fl_g, m_fl_b, m_fl_life);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		
	glBegin(GL_QUADS);
		glTexCoord2f(1.0f, 0.0f); glVertex3f(x[0], y[0], z[0]);
		glTexCoord2f(1.0f, 1.0f); glVertex3f(x[1], y[1], z[1]); 
		glTexCoord2f(0.0f, 1.0f); glVertex3f(x[2], y[2], z[2]);
		glTexCoord2f(0.0f, 0.0f); glVertex3f(x[3], y[3], z[3]);
	glEnd();

	glPopAttrib();
}


Particle::Particle(const Particle &p) :
	Object3(false, NULL, false),
	m_vec_movementdirection(p.m_vec_movementdirection),
	m_bo_physic(p.m_bo_physic),
	m_fl_life(p.m_fl_life),
	m_fl_fade(p.m_fl_fade),
	m_fl_r(p.m_fl_r),
	m_fl_g(p.m_fl_g),
	m_fl_b(p.m_fl_b),
	m_fl_size(p.m_fl_size),
	m_fl_growthRate(p.m_fl_growthRate),
	m_fl_weight(p.m_fl_weight),
	m_tex_particletexture(p.m_tex_particletexture)
{	/* nothing to do */	}


//////////////////////////////////// PARTICLE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	Unser Particle-Konstruktor, mit bergabeparametern.
/////
//////////////////////////////////// PARTICLE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
Particle::Particle(bool physic, float size, float growthRate, float weight, float fade, float red, float green, float blue)
{	
	m_bo_physic = physic;							// Make All The Particles Active
	
	m_fl_life		= 1.0f;							// Give All The Particles Full Life
	m_fl_fade		= fade;
	m_fl_size		= size;
	m_fl_growthRate = growthRate;
	m_fl_weight		= weight;

	boundingSphere(m_fl_size);

	m_fl_r = red;
	m_fl_g = green;
	m_fl_b = blue;
	
	m_vec_movementdirection = Vector(0.0, 0.0, 0.0);
}

//////////////////////////////////// PARTICLE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	Unser Particle-Konstruktor, ohne Farbewerte bei den bergabeparametern.
/////
//////////////////////////////////// PARTICLE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
Particle::Particle(bool physic, float size, float growthRate, float weight, float fade)
{
	m_bo_physic = physic;							// Make All The Particles Active
	
	m_fl_life		= 1.0f;							// Give All The Particles Full Life
	m_fl_fade		= fade;
	m_fl_size		= size;
	m_fl_growthRate = growthRate;
	m_fl_weight		= weight;

	boundingSphere(m_fl_size);

	m_fl_r = 1.0f;
	m_fl_g = 1.0f;
	m_fl_b = 1.0f;
	
	m_vec_movementdirection = Vector(0.0, 0.0, 0.0);
}


/////////////////////////////////// ~PARTICLE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	Unser Particle-Destruktor.
/////
/////////////////////////////////// ~PARTICLE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
Particle::~Particle()
{
}


//////////////////////////////////// UPDATE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	Diese Funktion aktualisiert die Position und die Blickrichtung
/////	des Partikels, sodass es immer zur Kamera schaut.
/////
//////////////////////////////////// UPDATE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
void Particle::cameraPosition(const Vector &camera)
{
	// Abstand zwischen der Kamera und dem Partikel.
	Vector	view = camera - getPosition();
	
	// Normalisieren den Blickvektors.
	view.normalise();

	phi = asin(-view.getY());
	theta = atan(view.getX() /view.getZ());
	if(view.getZ() < 0.0f)
		theta = (GLfloat)PI + theta;
}


bool Particle::advance(float time)
{
	Vector	pos = getPosition();

	// Wachstum des Partikels.
	m_fl_size += m_fl_growthRate * time;
	
	// Berechnen der neuen Position des Partikels, falls es sich um ein
	// aktives, also mobiles Element handelt.
	if(m_bo_physic)
	{
		pos += m_vec_movementdirection * time;
		pos.y() -= m_fl_weight * time;
	
		// Berechnung der Geschwindigkeitsverminderung des Partikels (= Abbremsung).
		m_vec_movementdirection /= pow(1.1f + fabs(m_fl_weight), time);
	
		// Berechnung der restlichen Lebenszeit des Partikels (abhngig von dessen Gre).
		m_fl_life -= (1 + m_fl_fade) * time / (8.0f * fabs(m_fl_weight));
	}
	else
	{
		pos += m_vec_movementdirection * time;

		// Berechnung der Geschwindigkeitsverminderung des Partikels (= Abbremsung).
		m_vec_movementdirection /= pow(1.3f, time);
		
		// Berechnung der restlichen Lebenszeit des Partikels.
		m_fl_life -= (1 + m_fl_fade) * time;
	}

	if(m_fl_life <= 0.0f)
		return false;
	else if(!(m_fl_life >= 0.0f))
		// invalid number - probalby divide by zero...
		return false;

	reset();
	postTranslate(pos);
	preRotateY(theta);
	preRotateX(phi);
	updateFinal();

	return true;
}


void Particle::glReload()
{
	Object3::glReload();

	if(!m_tex_particletexture.isEmpty())
		m_tex_particletexture->glReload();
}



GRAPHICS_END
