#include <graphics/Plant.h>

#include <graphics/Group3.h>
#include <graphics/Map.h>


GRAPHICS_BEGIN


Material	Plant::materials[10];
GLuint		Plant::nListID = 0;


void Plant::_load()
{	/* nothing to do */	}

void Plant::_remove()
{	/* nothing to do */	}

void Plant::_glLoad()
{
	if(nListID)
		return;

	nListID = glGenLists(1);

	glNewList(nListID, GL_COMPILE);
		glPushAttrib(GL_ENABLE_BIT);
		glDisable(GL_DEPTH_TEST);
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

		glColor4f(1.0f, 1.0f, 1.0f, 1.0f);

		glBegin(GL_QUADS);
			glNormal3f(0.0f, 0.7071f, 0.7071f);
			glTexCoord2f(1.0f, 1.0f);	glVertex3f( 0.5f, 0.0f, 0.0f);
			glTexCoord2f(1.0f, 0.0f);	glVertex3f( 0.5f, 1.0f, 0.0f); 
			glTexCoord2f(0.0f, 0.0f);	glVertex3f(-0.5f, 1.0f, 0.0f);
			glTexCoord2f(0.0f, 1.0f);	glVertex3f(-0.5f, 0.0f, 0.0f);
			
			glNormal3f(0.0f, 0.7071f, -0.7071f);
			glTexCoord2f(0.0f, 1.0f);	glVertex3f(-0.5f, 0.0f, 0.0f);
			glTexCoord2f(0.0f, 0.0f);	glVertex3f(-0.5f, 1.0f, 0.0f); 
			glTexCoord2f(1.0f, 0.0f);	glVertex3f( 0.5f, 1.0f, 0.0f);
			glTexCoord2f(1.0f, 1.0f);	glVertex3f( 0.5f, 0.0f, 0.0f);

			glNormal3f(0.7071f, 0.7071f, 0.0f);
			glTexCoord2f(1.0f, 1.0f);	glVertex3f(0.0f, 0.0f,  0.5f);
			glTexCoord2f(1.0f, 0.0f);	glVertex3f(0.0f, 1.0f,  0.5f);
			glTexCoord2f(0.0f, 0.0f);	glVertex3f(0.0f, 1.0f, -0.5f);
			glTexCoord2f(0.0f, 1.0f);	glVertex3f(0.0f, 0.0f, -0.5f);

			glNormal3f(-0.7071f, 0.7071f, 0.0f);
			glTexCoord2f(1.0f, 1.0f);	glVertex3f(0.0f, 0.0f, -0.5f);
			glTexCoord2f(1.0f, 0.0f);	glVertex3f(0.0f, 1.0f, -0.5f);
			glTexCoord2f(0.0f, 0.0f);	glVertex3f(0.0f, 1.0f,  0.5f);
			glTexCoord2f(0.0f, 1.0f);	glVertex3f(0.0f, 0.0f,  0.5f);
		glEnd();

		glPopAttrib();
	glEndList();
}

void Plant::_glRemove()
{	glDeleteLists(nListID, 1);
	nListID = 0;				}


void Plant::_glRender()
{	glCallList(nListID);	}


Plant::Plant(const float &x, const float &y, const float &z, const float &size, const float &angle)
{
	reset();
	postRotateY(angle);
	postScale(size, size, size);
	postTranslate(x, y, z);
	updateFinal();
}

Plant::Plant(const Vector &position, const float &size, const float &angle)
{
	reset();
	postRotateY(angle);
	postScale(size, size, size);
	postTranslate(position);
	updateFinal();
}

Group3 * Plant::createField(const Plant::Vector &position, const uint_t &id, const float &size, const float &sizex, const float &sizez, const uint_t &count, const Map &terrain)
{
	Group3 *plants = new Group3();
	plants->deleteOnRemove(true);

	float	s, x, y, z,
			minx = position.getX(),
			maxx = minx,
			minz = position.getZ(),
			maxz = minz,
			miny = terrain.heightAt(minx, minz),
			maxy = miny;
	Plant	*p;

	plants->postTranslate(position.getX(), 0.0f, position.getZ());

	for(uint_t i = 0; i < count; i++)
	{
		// Zufllige Positionierung der Pflanze.
		x = position.getX() + sizex * ((float)rand() / (RAND_MAX >> 1) - 1.0f);
		z = position.getZ() + sizez * ((float)rand() / (RAND_MAX >> 1) - 1.0f);

		y = terrain.heightAt(x, z);

		if(x < minx)
			minx = x;
		else if(x > maxx)
			maxx = x;
		if(y < miny)
			miny = y;
		else if(y > maxy)
			maxy = y;
		if(z < minz)
			minz = z;
		else if(z > maxz)
			maxz = z;

		// Zufllige Gre.
		s = size + 0.5f * ((float)rand() / RAND_MAX - 0.5f);

		p = new Plant(x, y, z, s, 2.0f * (float)PI * ((float)rand() / RAND_MAX));
		p->deleteOnRemove(true);
		p->material(materials + id);
		plants->add(p);
	}

	plants->reset();
	plants->postTranslate((minx + maxx) / 2.0f, (miny + maxy + size) / 2.0f, (minz + maxz) / 2.0f);
	plants->updateFinal();

	plants->boundingBox(maxx + size / 2.0f - minx, maxy + size - miny, maxz + size / 2.0f - minz);
	plants->visibleRange(sqrt(size) * 150.0f);

	return plants;
}

Group3 * Plant::createPlants(const Plant::Vector &position, const uint_t &id, const float &size, const float &radius, const uint_t &count, const Map &terrain)
{
	Group3 *plants = new Group3();
	plants->deleteOnRemove(true);

	float	a, r, s, x, y, z,
			minx = position.getX(),
			maxx = minx,
			minz = position.getZ(),
			maxz = minz,
			miny = terrain.heightAt(minx, minz),
			maxy = miny;
	Plant	*p;
	
	for(uint_t i = 0; i < count; i++)
	{
		// Zufllige Positionierung der Pflanze.
		r = radius * ((float)rand() / RAND_MAX);
		a = 2.0f * (float)PI * ((float)rand() / RAND_MAX);
		
		x = position.getX() + r * cos(a);
		z = position.getZ() + r * sin(a);

		y = terrain.heightAt(x, z);
		
		if(x < minx)
			minx = x;
		else if(x > maxx)
			maxx = x;
		if(y < miny)
			miny = y;
		else if(y > maxy)
			maxy = y;
		if(z < minz)
			minz = z;
		else if(z > maxz)
			maxz = z;

		// Zufllige Gre.
		s = size * (1.3f - pow(cos(((float)(rand() % 100 + 1)) / 100.0f * (float)(PI / 2.0f)), 0.2f));

		p = new Plant(x, y, z, s, 2.0f * (float)PI * ((float)rand() / RAND_MAX));
		p->deleteOnRemove(true);
		p->material(materials + id);
		plants->add(p);
	}

	plants->reset();
	plants->postTranslate((minx + maxx) / 2.0f, (miny + maxy + size) / 2.0f, (minz + maxz) / 2.0f);
	plants->updateFinal();

	plants->boundingBox(maxx + size / 2.0f - minx, maxy + size - miny, maxz + size / 2.0f - minz);
	plants->visibleRange(sqrt(size) * 150.0f);

	return plants;
}


void Plant::glReload()
{
	Object3::glReload();

	nListID = 0;
	for(uint_t i = 0; i < 10; i++)
		materials[i].glReload();
}


GRAPHICS_END
