#include <graphics/SDLSemaphore.h>


GRAPHICS_BEGIN


SDLSemaphore::Allocator	SDLSemaphore::allocator;


SDL_sem * SDLSemaphore::Allocator::_new() const
{
	SDL_sem	*sdlSemaphore = SDL_CreateSemaphore(0);
	if(!sdlSemaphore)
		throw SDLException(getLastSDLError());
	return sdlSemaphore;
}

void SDLSemaphore::Allocator::_delete(SDL_sem *sdlSemaphore) const
{	SDL_DestroySemaphore(sdlSemaphore);	}


SDLSemaphore::~SDLSemaphore()
{	/* nothing to do */	}


void SDLSemaphore::wait()
{
	if(sdlSemaphore.isEmpty())
		sdlSemaphore.create();
	if(SDL_SemWait(*this))
		throw SDLException(getLastSDLError());
}

bool SDLSemaphore::tryWait()
{
	if(sdlSemaphore.isEmpty())
		sdlSemaphore.create();

	int_t	nCode = SDL_SemTryWait(*this);

	if(!nCode)
		return 1;
	else if(nCode == SDL_MUTEX_TIMEDOUT)
		return 0;

	throw SDLException(getLastSDLError());
}

void SDLSemaphore::wait(uint32_t nTimeoutMs)
{
	if(sdlSemaphore.isEmpty())
		sdlSemaphore.create();

	int_t	nCode = SDL_SemWaitTimeout(*this, nTimeoutMs);

	if(nCode == SDL_MUTEX_TIMEDOUT)
		throw TimeoutException();
	else if(nCode)
		throw SDLException(getLastSDLError());
}

void SDLSemaphore::post()
{
	if(sdlSemaphore.isEmpty())
		sdlSemaphore.create();
	if(SDL_SemPost(*this))
		throw SDLException(getLastSDLError());
}


GRAPHICS_END
