#include "SimulationServer.h"


GRAPHICS_BEGIN


void SimulationServer::_initialise()
{
	initialise();
	state = loading;
}

void SimulationServer::_load()
{
	load();
	state = running;
}

void SimulationServer::load()
{	/* nothing to do */	}


void SimulationServer::_simulate(float time)
{
	clients.insert(newClients.begin(), newClients.end());
	newClients.clear();

	for(Clients::iterator it = clients.begin(); it != clients.end();)
		if(!(*it)->advance(time))
		{
			if((*it)->getAutoDelete())
				delete *it;
			it = clients.erase(it);
		}
		else
			it++;

	simulate(time);
}

void SimulationServer::pause(float time)
{	/* nothing to do */	}


void SimulationServer::terminate()
{
	state = terminated;
}


void SimulationServer::run(uint32_t _nTimeMs)
{
	switch(state)
	{
	case initialising:
		_initialise();
	case loading:
		_load();
		break;

	case running:
		_simulate(_nTimeMs / 1000.0f);
		break;
	case paused:
		pause(_nTimeMs / 1000.0f);
		break;

	case terminating:
		terminate();
		cancel();
		break;

	default:
		throw Exception(getErrorMessage("unhandled state"));
		break;
	}
}


SimulationServer::SimulationServer() :
	state(initialising)
{
}

SimulationServer::~SimulationServer()
{
}


void SimulationServer::add(SimulationClient *sc)
{
	sc->server(this);
	newClients.insert(sc);
}


GRAPHICS_END
