#include <graphics/Sounds.h>

#include <fmod/fmod_errors.h>


GRAPHICS_BEGIN


ObjectManager<FSOUND_SAMPLE, string, Sound::Allocator>	Sound::samples;


FSOUND_SAMPLE * Sound::Allocator::_new(const string *sample) const
{
	FSOUND_SAMPLE *pSample = FSOUND_Sample_Load(FSOUND_UNMANAGED, sample->c_str(), FSOUND_HW3D, 0, 0);
    if(!pSample)
		throw Exception(getErrorMessage(FMOD_ErrorString(FSOUND_GetError())));

	return pSample;
}

void Sound::Allocator::_delete(FSOUND_SAMPLE *pSample) const
{	FSOUND_Sample_Free(pSample);	}

		
///////////////////////////// SOUND OBJECT VEHICLE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	VehicleTyp: 1 = Small, 2 = Medium, 3 = Heavy.
/////
///////////////////////////// SOUND OBJECT VEHICLE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
Sound::Sound(const string &type, bool loop, bool restartable) :
	strSample(type),
	m_bo_loop(loop),
	m_bo_restartable(restartable)
{
}


///////////////////////////// ~SOUND OBJECT VEHICLE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	Unser Standard-SoundObject_Vehicle-Destruktor.
/////
///////////////////////////// ~SOUND OBJECT VEHICLE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
Sound::~Sound()
{	/* nothing to do */	}


void Sound::initialise()
{
	if (FSOUND_GetVersion() < FMOD_VERSION)
		throw getErrorMessage("FMOD DLL is out of date");

	// Output Method: Direct Sound.
	FSOUND_SetOutput(FSOUND_OUTPUT_DSOUND);

	// Driver: Direct Sound.
	FSOUND_SetDriver(1);

	// Mixer: Autodetect.
	FSOUND_SetMixer(FSOUND_MIXER_AUTODETECT);

	// SpeakerMode: DolbyDigital.
	FSOUND_SetSpeakerMode(FSOUND_SPEAKERMODE_DOLBYDIGITAL);
	
	// Initialize FMOD.
	if (!FSOUND_Init(44100, 16, 0))
		throw getErrorMessage(FMOD_ErrorString(FSOUND_GetError()));
}

//////////////////////////// START EXPLOSION SOUND \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	Diese Funktion startet einen Sound, nach dem Prinzip "Fire And Forget".
/////	Nach der Sound einnmal gestartet wurde muss man sich nicht mehr darum kmmern.
/////
//////////////////////////// START EXPLOSION SOUND \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
void Sound::start(bool bPosition)
{	
	if(sample.isEmpty())
	{
		sample = samples.create(strSample);

		// Durch das Erhhen der Mindestdistanz wird die Lautstrke lauter.
		FSOUND_Sample_SetMinMaxDistance(sample.getPointer(), 4.0f, 100.0f);
		if(m_bo_loop)
			FSOUND_Sample_SetMode(sample.getPointer(), FSOUND_LOOP_NORMAL);
	}
	else if(!m_bo_restartable)
		if(FSOUND_IsPlaying(m_int_channel))
			return;

	// Durch das Erhhen der Mindestdistanz wird die Lautstrke lauter.
    FSOUND_Sample_SetMinMaxDistance(sample.getPointer(), 4.0f, 10000.0f);  
	
	m_int_channel	= FSOUND_PlaySound(FSOUND_FREE, sample.getPointer());
	m_int_volume	= FSOUND_GetVolume(m_int_channel);
	m_int_frequency	= FSOUND_GetFrequency(m_int_channel);

	if(bPosition)
		FSOUND_3D_SetAttributes(m_int_channel, m_fl_position, NULL);

	FSOUND_Update();
}


void Sound::stop()
{
	if(m_int_channel)
		FSOUND_StopSound(m_int_channel);
}


////////////////////////////  \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	Diese Funktion startet einen Sound, nach dem Prinzip "Fire And Forget".
/////	Nach der Sound einnmal gestartet wurde muss man sich nicht mehr darum kmmern.
/////
//////////////////////////// START EXPLOSION SOUND \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
void Sound::update()
{
	FSOUND_SetFrequency( m_int_channel, m_int_frequency );
	FSOUND_SetVolume( m_int_channel, m_int_volume);
	FSOUND_3D_SetAttributes( m_int_channel, m_fl_position, NULL );

	FSOUND_Update();
}

void Sound::pause(bool pause)
{
	FSOUND_SetPaused( m_int_channel, pause );
}


GRAPHICS_END
