#include <graphics/Sunray.h>

#include <graphics/Group3.h>


GRAPHICS_BEGIN


Texture::Reference	Sunray::rTexture;
Sunray::Vector		Sunray::vDirection,
					Sunray::vPosition;
float				Sunray::r = 1.0f,
					Sunray::g = 1.0f,
					Sunray::b = 0.8f,
					Sunray::fDepth,
					Sunray::fLength = 50.0f,
					Sunray::fTurbolence = 1.0f;


void Sunray::_load()
{	/* nothing to do */	}

void Sunray::_remove()
{	/* nothing to do */	}

void Sunray::_glLoad()
{	/* nothing to do */	}

void Sunray::_glRemove()
{	/* nothing to do */	}


void Sunray::_glRender()
{
	glPushAttrib(GL_ENABLE_BIT);
	glDisable(GL_DEPTH_TEST);
	glDisable(GL_LIGHTING);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	glColor4f(r, g, b, 0.5f * (1.0f - pow(1.0f - (float)sin(age * PI), 0.2f)));
	
	glBegin(GL_QUADS);
		glTexCoord2f(1.0f, 0.99f);	glVertex3f( 2.5f * vz + vDirection.getX() * fLength,	vDirection.getY() * fLength,	-2.5f * vx + vDirection.getZ() * fLength);
		glTexCoord2f(1.0f, 0.0f);	glVertex3f( 2.5f * vz,									0.0f,							-2.5f * vx); 
		glTexCoord2f(0.0f, 0.0f);	glVertex3f(-2.5f * vz,									0.0f,							 2.5f * vx);
		glTexCoord2f(0.0f, 0.99f);	glVertex3f(-2.5f * vz + vDirection.getX() * fLength,	vDirection.getY() * fLength,	 2.5f * vx + vDirection.getZ() * fLength);
	glEnd();

	glPopAttrib();
}


Group3 * Sunray::create(const uint_t &count)
{
	Group3	*rays = new Group3();
	rays->deleteOnRemove(true);

	Sunray	*s;
	for(uint_t i = 0; i < count; i++)
	{
		s = new Sunray();
		s->deleteOnRemove(true);
		s->advance(7.5f * (1.0f + (float)rand() / RAND_MAX));
		rays->add(s);
	}

	return rays;
}


void Sunray::cameraPosition(const Vector &camera)
{
	Vector	v(camera - getGlPosition() - vDirection / abs(vDirection.getY()) * (fDepth - camera.getY()));

	v.normalise();
	vx = v.getX();
	vz = v.getZ();
}


bool Sunray::advance(float time)
{
	// Falls Sonnenstrahl erlischt wird er neu um die Kamera positioniert.
	age += time * fTurbolence / 7.5f;

	if(age >= 1.0f)
	{
		age -= 1.0f;

		float	r = 400.0f * ((float)rand() / RAND_MAX),
				a = 2.0f * (float)PI * ((float)rand() / RAND_MAX),
				x = vPosition.getX() + r * cos(a),
				z = vPosition.getZ() + r * sin(a);

		reset();
		postTranslate(x, fDepth, z);
		updateFinal();
	}

	return true;
}


void Sunray::glReload()
{
	Object3::glReload();

	if(!rTexture.isEmpty())
		rTexture->glReload();
}


void Sunray::glBind()
{	if(!rTexture.isEmpty())
		rTexture->glBindLoad();	}


GRAPHICS_END
